@@ -5,7 +5,7 @@
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 /**
  * SECTION:base64
@@ -736,98 +736,6 @@ xmlSecBase64DecodeInPlace(xmlChar* str, xmlSecSize* outWritten) {
     return(xmlSecBase64Decode_ex(str, (xmlSecByte*)str, xmlSecStrlen(str) + 1,outWritten));
 }
 
-/**************************************************************
- *
- * Deprecated functions for backward compatibility
- *
- **************************************************************/
-
-/**
- * xmlSecBase64CtxUpdate:
- * @ctx:                the pointer to #xmlSecBase64Ctx structure
- * @in:                 the input buffer
- * @inSize:             the input buffer size
- * @out:                the output buffer
- * @outSize:            the output buffer size
- *
- * DEPRECATED. Encodes or decodes the next piece of data from input buffer.
- *
- * Returns: the number of bytes written to output buffer or
- * -1 if an error occurs.
- */
-int
-xmlSecBase64CtxUpdate(xmlSecBase64CtxPtr ctx, const xmlSecByte* in, xmlSecSize inSize,
-    xmlSecByte* out, xmlSecSize outSize) {
-
-    int ret;
-    xmlSecSize outWritten;
-    int res;
-
-    ret = xmlSecBase64CtxUpdate_ex(ctx, in, inSize, out, outSize, &outWritten);
-    if (ret < 0) {
-        return(-1);
-    }
-
-    XMLSEC_SAFE_CAST_SIZE_TO_INT(outWritten, res, return(-1), NULL);
-    return(res);
-}
-
-
-/**
- * xmlSecBase64CtxFinal:
- * @ctx:                the pointer to #xmlSecBase64Ctx structure
- * @out:                the output buffer
- * @outSize:            the output buffer size
- *
- * DEPRECATED. Encodes or decodes the last piece of data stored in the context
- * and finalizes the result.
- *
- * Returns: the number of bytes written to output buffer or
- * -1 if an error occurs.
- */
-int
-xmlSecBase64CtxFinal(xmlSecBase64CtxPtr ctx, xmlSecByte* out, xmlSecSize outSize) {
-    int ret;
-    xmlSecSize outWritten;
-    int res;
-
-    ret = xmlSecBase64CtxFinal_ex(ctx, out, outSize, &outWritten);
-    if (ret < 0) {
-        return(-1);
-    }
-
-    XMLSEC_SAFE_CAST_SIZE_TO_INT(outWritten, res, return(-1), NULL);
-    return(res);
-}
-
- /**
-  * xmlSecBase64Decode:
-  * @str:                the input buffer with base64 encoded string
-  * @out:                the output buffer
-  * @outSize:            the output buffer size
-  *
-  * DEPRECATED. Decodes input base64 encoded string and puts result into
-  * the output buffer.
-  *
-  * Returns: the number of bytes written to the output buffer or
-  * a negative value if an error occurs
-  */
-int
-xmlSecBase64Decode(const xmlChar* str, xmlSecByte* out, xmlSecSize outSize) {
-    int ret;
-    xmlSecSize outWritten;
-    int res;
-
-    ret = xmlSecBase64Decode_ex(str, out, outSize, &outWritten);
-    if (ret < 0) {
-        return(-1);
-    }
-
-    XMLSEC_SAFE_CAST_SIZE_TO_INT(outWritten, res, return(-1), NULL);
-    return(res);
-}
-
-
 /**************************************************************
  *
  * Base64 Transform
