@@ -12,6 +12,7 @@
  * @Short_description: Application support functions for Microsoft Cryptography API: Next Generation (CNG).
  * @Stability: Stable
  *
+ * Common functions for xmlsec1 command line utility tool for Microsoft Cryptography API: Next Generation (CNG).
  */
 
 #include "globals.h"
@@ -22,19 +23,21 @@
 
 #include <xmlsec/xmlsec.h>
 #include <xmlsec/keys.h>
-#include <xmlsec/transforms.h>
 #include <xmlsec/errors.h>
 #include <xmlsec/keysmngr.h>
+#include <xmlsec/private.h>
+#include <xmlsec/transforms.h>
 #include <xmlsec/xmltree.h>
 
 #include <xmlsec/mscng/app.h>
 #include <xmlsec/mscng/crypto.h>
-#include <xmlsec/mscng/symbols.h>
-#include <xmlsec/mscng/x509.h>
 #include <xmlsec/mscng/certkeys.h>
 #include <xmlsec/mscng/keysstore.h>
+#include <xmlsec/mscng/symbols.h>
+#include <xmlsec/mscng/x509.h>
 
 #include "../cast_helpers.h"
+#include "private.h"
 
 /* config info for the mscng keysstore */
 static LPTSTR gXmlSecMSCngAppCertStoreName = NULL;
@@ -113,21 +116,40 @@ xmlSecMSCngAppGetCertStoreName(void) {
  * @pwdCallback:        the key password callback.
  * @pwdCallbackCtx:     the user context for password callback.
  *
- * Reads key from the a file (not implemented yet).
- *
+ * Deprecated, use @xmlSecMSCngAppKeyLoadEx instead. Reads key from the a file.
  * Returns: pointer to the key or NULL if an error occurs.
  */
 xmlSecKeyPtr
 xmlSecMSCngAppKeyLoad(const char *filename, xmlSecKeyDataFormat format,
-                      const char *pwd,
-                      void* pwdCallback,
-                      void* pwdCallbackCtx) {
+    const char *pwd, void* pwdCallback, void* pwdCallbackCtx
+) {
+    return(xmlSecMSCngAppKeyLoadEx(filename,  xmlSecKeyDataTypeUnknown, format, pwd, pwdCallback, pwdCallbackCtx));
+}
+
+/**
+ * xmlSecMSCngAppKeyLoadEx:
+ * @filename:           the key filename.
+ * @type:               the expected key type.
+ * @format:             the key file format.
+ * @pwd:                the key file password.
+ * @pwdCallback:        the key password callback.
+ * @pwdCallbackCtx:     the user context for password callback.
+ *
+ * Reads key from the a file.
+ *
+ * Returns: pointer to the key or NULL if an error occurs.
+ */
+xmlSecKeyPtr
+xmlSecMSCngAppKeyLoadEx(const char *filename, xmlSecKeyDataType type ATTRIBUTE_UNUSED, xmlSecKeyDataFormat format,
+    const char *pwd, void* pwdCallback, void* pwdCallbackCtx
+) {
     xmlSecBuffer buffer;
     xmlSecKeyPtr key = NULL;
     int ret;
 
     xmlSecAssert2(filename != NULL, NULL);
     xmlSecAssert2(format != xmlSecKeyDataFormatUnknown, NULL);
+    UNREFERENCED_PARAMETER(type);
 
     switch(format) {
     case xmlSecKeyDataFormatPkcs12:
@@ -190,7 +212,6 @@ xmlSecKeyPtr
 xmlSecMSCngAppKeyLoadMemory(const xmlSecByte* data, xmlSecSize dataSize, xmlSecKeyDataFormat format,
                             const char *pwd, void* pwdCallback, void* pwdCallbackCtx) {
     PCCERT_CONTEXT pCert = NULL;
-    PCCERT_CONTEXT pCertChain = NULL;
     PCCERT_CONTEXT pKeyCert = NULL;
     xmlSecKeyDataPtr x509Data = NULL;
     xmlSecKeyDataPtr keyData = NULL;
@@ -206,20 +227,13 @@ xmlSecMSCngAppKeyLoadMemory(const xmlSecByte* data, xmlSecSize dataSize, xmlSecK
     UNREFERENCED_PARAMETER(pwdCallback);
     UNREFERENCED_PARAMETER(pwdCallbackCtx);
 
-    /* read cert and make a copy for cert chain and keyCert */
+    /* read cert and make a copy for keyCert */
     XMLSEC_SAFE_CAST_SIZE_TO_ULONG(dataSize, dwDataSize, goto done, NULL);
     pCert = CertCreateCertificateContext(X509_ASN_ENCODING | PKCS_7_ASN_ENCODING, data, dwDataSize);
     if(pCert == NULL) {
         xmlSecMSCngLastError("CertCreateCertificateContext", NULL);
         goto done;
     }
-
-    pCertChain = CertDuplicateCertificateContext(pCert);
-    if(pCertChain == NULL) {
-        xmlSecMSCngLastError("CertDuplicateCertificateContext", NULL);
-        goto done;
-    }
-
     pKeyCert = CertDuplicateCertificateContext(pCert);
     if(pKeyCert == NULL) {
         xmlSecMSCngLastError("CertDuplicateCertificateContext", NULL);
@@ -248,7 +262,7 @@ xmlSecMSCngAppKeyLoadMemory(const xmlSecByte* data, xmlSecSize dataSize, xmlSecK
     }
     keyData = NULL;
 
-   /* add cert and keyCert to x509 data and add it to the key */
+    /* add keyCert to x509 data and add it to the key */
     x509Data = xmlSecKeyDataCreate(xmlSecMSCngKeyDataX509Id);
     if(x509Data == NULL) {
         xmlSecInternalError("xmlSecKeyDataCreate", NULL);
@@ -262,13 +276,6 @@ xmlSecMSCngAppKeyLoadMemory(const xmlSecByte* data, xmlSecSize dataSize, xmlSecK
     }
     pKeyCert = NULL; /* owned by x509Data data now */
 
-    ret = xmlSecMSCngKeyDataX509AdoptCert(x509Data, pCertChain);
-    if(ret < 0) {
-        xmlSecInternalError("xmlSecMSCngKeyDataX509AdoptCert", NULL);
-        goto done;
-    }
-    pCertChain = NULL; /* owned by x509Data data now */
-
     ret = xmlSecKeyAdoptData(key, x509Data);
     if(ret < 0) {
         xmlSecInternalError("xmlSecKeyAdoptData", NULL);
@@ -284,9 +291,6 @@ done:
     if(pCert != NULL) {
         CertFreeCertificateContext(pCert);
     }
-    if(pCertChain != NULL) {
-        CertFreeCertificateContext(pCertChain);
-    }
     if(pKeyCert != NULL) {
         CertFreeCertificateContext(pKeyCert);
     }
@@ -311,8 +315,7 @@ done:
  * @filename:           the certificate filename.
  * @format:             the certificate file format.
  *
- * Reads the certificate from $@filename and adds it to key
- * (not implemented yet).
+ * Reads the certificate from $@filename and adds it to key.
  *
  * Returns: 0 on success or a negative value otherwise.
  */
@@ -359,9 +362,9 @@ xmlSecMSCngAppKeyCertLoadMemory(xmlSecKeyPtr key, const xmlSecByte* data, xmlSec
  * @pwdCallback:        the password callback.
  * @pwdCallbackCtx:     the user context for password callback.
  *
- * Reads key and all associated certificates from the PKCS12 file
- * (not implemented yet).
- * For uniformity, call xmlSecMSCngAppKeyLoad instead of this function. Pass
+ * Reads key and all associated certificates from the PKCS12 file.
+ *
+ * For uniformity, call @xmlSecMSCngAppKeyLoadEx instead of this function. Pass
  * in format=xmlSecKeyDataFormatPkcs12.
  *
  *
@@ -369,9 +372,8 @@ xmlSecMSCngAppKeyCertLoadMemory(xmlSecKeyPtr key, const xmlSecByte* data, xmlSec
  */
 xmlSecKeyPtr
 xmlSecMSCngAppPkcs12Load(const char *filename,
-                         const char *pwd,
-                         void* pwdCallback,
-                         void* pwdCallbackCtx) {
+    const char *pwd, void* pwdCallback, void* pwdCallbackCtx
+) {
     xmlSecBuffer buffer;
     xmlSecByte* data;
     xmlSecKeyPtr key;
@@ -421,7 +423,7 @@ xmlSecMSCngAppPkcs12Load(const char *filename,
  * @pwdCallbackCtx:     the user context for password callback.
  *
  * Reads key and all associated certificates from the PKCS12 binary data.
- * For uniformity, call xmlSecMSCngAppKeyLoad instead of this function. Pass
+ * For uniformity, call @xmlSecMSCngAppKeyLoadEx instead of this function. Pass
  * in format=xmlSecKeyDataFormatPkcs12.
  *
  * Returns: pointer to the key or NULL if an error occurs.
@@ -440,6 +442,7 @@ xmlSecMSCngAppPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize, cons
     xmlSecKeyDataPtr privKeyData = NULL;
     PCCERT_CONTEXT cert = NULL;
     PCCERT_CONTEXT certDuplicate = NULL;
+    xmlChar* keyName = NULL;
     int ret;
 
     xmlSecAssert2(data != NULL, NULL);
@@ -468,7 +471,7 @@ xmlSecMSCngAppPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize, cons
         goto cleanup;
     }
 
-    DWORD flags = CRYPT_EXPORTABLE;
+    DWORD flags = CRYPT_EXPORTABLE | PKCS12_PREFER_CNG_KSP;
     if (!xmlSecImportGetPersistKey()) {
         flags |= PKCS12_NO_PERSIST_KEY;
     }
@@ -492,6 +495,16 @@ xmlSecMSCngAppPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize, cons
         ret = CertGetCertificateContextProperty(cert, CERT_KEY_SPEC_PROP_ID,
             &dwData, &dwDataLen);
         if(ret == TRUE) {
+            if (privKeyData != NULL) {
+                /* multiple private keys, use the first one */
+                continue;
+            }
+
+            /* get key name */
+            if (keyName == NULL) {
+                keyName = xmlSecMSCngX509GetFriendlyNameUtf8(cert);
+            }
+
             /* adopt private key */
             certDuplicate = CertDuplicateCertificateContext(cert);
             if(certDuplicate == NULL) {
@@ -506,21 +519,35 @@ xmlSecMSCngAppPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize, cons
                 goto cleanup;
             }
             certDuplicate = NULL;
-        }
 
-        /* adopt certificate */
-        certDuplicate = CertDuplicateCertificateContext(cert);
-        if(certDuplicate == NULL) {
-            xmlSecMSCngLastError("CertDuplicateCertificateContext", NULL);
-            goto cleanup;
-        }
+            /* adopt key certificate */
+            certDuplicate = CertDuplicateCertificateContext(cert);
+            if (certDuplicate == NULL) {
+                xmlSecMSCngLastError("CertDuplicateCertificateContext", NULL);
+                goto cleanup;
+            }
 
-        ret = xmlSecMSCngKeyDataX509AdoptCert(keyData, certDuplicate);
-        if(ret < 0) {
-            xmlSecInternalError("xmlSecMSCngKeyDataX509AdoptKeyCert", NULL);
-            goto cleanup;
+            ret = xmlSecMSCngKeyDataX509AdoptKeyCert(keyData, certDuplicate);
+            if (ret < 0) {
+                xmlSecInternalError("xmlSecMSCngKeyDataX509AdoptKeyCert", NULL);
+                goto cleanup;
+            }
+            certDuplicate = NULL;
+        } else {
+            /* adopt certificate */
+            certDuplicate = CertDuplicateCertificateContext(cert);
+            if (certDuplicate == NULL) {
+                xmlSecMSCngLastError("CertDuplicateCertificateContext", NULL);
+                goto cleanup;
+            }
+
+            ret = xmlSecMSCngKeyDataX509AdoptCert(keyData, certDuplicate);
+            if (ret < 0) {
+                xmlSecInternalError("xmlSecMSCngKeyDataX509AdoptKeyCert", NULL);
+                goto cleanup;
+            }
+            certDuplicate = NULL;
         }
-        certDuplicate = NULL;
     }
 
     /* at this point we should have a private key */
@@ -553,6 +580,15 @@ xmlSecMSCngAppPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize, cons
     }
     keyData = NULL;
 
+    if (keyName != NULL) {
+        ret = xmlSecKeySetName(key, keyName);
+        if (ret < 0) {
+            xmlSecInternalError("xmlSecKeySetName", NULL);
+            xmlSecKeyDestroy(key);
+            goto cleanup;
+        }
+    }
+
 cleanup:
     if(certStore != NULL) {
         CertCloseStore(certStore, 0);
@@ -560,6 +596,9 @@ cleanup:
     if(pwdWideChar != NULL) {
         xmlFree(pwdWideChar);
     }
+    if (keyName != NULL) {
+        xmlFree(keyName);
+    }
     if(keyData != NULL) {
         xmlSecKeyDataDestroy(keyData);
     }
@@ -581,14 +620,14 @@ cleanup:
  *                      trusted or not.
  *
  * Reads cert from @filename and adds to the list of trusted or known
- * untrusted certs in @store (not implemented yet).
+ * untrusted certs in @store.
  *
  * Returns: 0 on success or a negative value otherwise.
  */
 int
 xmlSecMSCngAppKeysMngrCertLoad(xmlSecKeysMngrPtr mngr, const char *filename,
                                xmlSecKeyDataFormat format,
-                               xmlSecKeyDataType type ATTRIBUTE_UNUSED) {
+                               xmlSecKeyDataType type) {
     xmlSecBuffer buffer;
     int ret;
 
@@ -626,8 +665,8 @@ xmlSecMSCngAppKeysMngrCertLoad(xmlSecKeysMngrPtr mngr, const char *filename,
 /**
  * xmlSecMSCngAppKeysMngrCertLoadMemory:
  * @mngr:               the pointer to keys manager.
- * @data:               the key binary data.
- * @dataSize:           the key binary data size.
+ * @data:               the certificate data.
+ * @dataSize:           the certificate data size.
  * @format:             the certificate format (PEM or DER).
  * @type:               the certificate type (trusted/untrusted).
  *
@@ -638,8 +677,8 @@ xmlSecMSCngAppKeysMngrCertLoad(xmlSecKeysMngrPtr mngr, const char *filename,
  */
 int
 xmlSecMSCngAppKeysMngrCertLoadMemory(xmlSecKeysMngrPtr mngr, const xmlSecByte* data,
-                                     xmlSecSize dataSize, xmlSecKeyDataFormat format,
-                                     xmlSecKeyDataType type) {
+    xmlSecSize dataSize, xmlSecKeyDataFormat format, xmlSecKeyDataType type
+) {
     xmlSecKeyDataStorePtr x509Store;
     PCCERT_CONTEXT pCert = NULL;
     DWORD dwDataSize;
@@ -685,6 +724,51 @@ xmlSecMSCngAppKeysMngrCertLoadMemory(xmlSecKeysMngrPtr mngr, const xmlSecByte* d
     return(0);
 }
 
+/**
+ * xmlSecMSCngAppKeysMngrCrlLoad:
+ * @mngr:               the keys manager.
+ * @filename:           the CRL file.
+ * @format:             the CRL file format.
+ *
+ * Reads crls from @filename and adds to the list of crls in @store.
+ *
+ * Returns: 0 on success or a negative value otherwise.
+ */
+int
+xmlSecMSCngAppKeysMngrCrlLoad(xmlSecKeysMngrPtr mngr, const char *filename, xmlSecKeyDataFormat format) {
+    xmlSecAssert2(mngr != NULL, -1);
+    xmlSecAssert2(filename != NULL, -1);
+    xmlSecAssert2(format != xmlSecKeyDataFormatUnknown, -1);
+
+    /* TODO */
+    xmlSecNotImplementedError(NULL);
+    return(-1);
+}
+
+/**
+ * xmlSecMSCngAppKeysMngrCrlLoadMemory:
+ * @mngr:               the pointer to keys manager.
+ * @data:               the CRL data.
+ * @dataSize:           the CRL data size.
+ * @format:             the CRL format (PEM or DER).
+ *
+ * Reads crls from @data and adds to the list of crls in @store
+ *
+ * Returns: 0 on success or a negative value otherwise.
+ */
+int
+xmlSecMSCngAppKeysMngrCrlLoadMemory(xmlSecKeysMngrPtr mngr, const xmlSecByte* data, xmlSecSize dataSize, xmlSecKeyDataFormat format) {
+    xmlSecAssert2(mngr != NULL, -1);
+    xmlSecAssert2(data != NULL, -1);
+    xmlSecAssert2(dataSize > 0, -1);
+    xmlSecAssert2(format != xmlSecKeyDataFormatUnknown, -1);
+
+    /* TODO */
+    xmlSecNotImplementedError(NULL);
+    return(-1);
+}
+
+
 #endif /* XMLSEC_NO_X509 */
 
 /**
@@ -763,6 +847,33 @@ xmlSecMSCngAppDefaultKeysMngrAdoptKey(xmlSecKeysMngrPtr mngr, xmlSecKeyPtr key)
     return(0);
 }
 
+/**
+ * xmlSecMSCngAppDefaultKeysMngrVerifyKey:
+ * @mngr:               the pointer to keys manager.
+ * @key:                the pointer to key.
+ * @keyInfoCtx:         the key info context for verification.
+ *
+ * Verifies @key with the keys manager @mngr created with #xmlSecCryptoAppDefaultKeysMngrInit
+ * function:
+ * - Checks that key certificate is present
+ * - Checks that key certificate is valid
+ *
+ * Adds @key to the keys manager @mngr created with #xmlSecCryptoAppDefaultKeysMngrInit
+ * function.
+ *
+ * Returns: 1 if key is verified, 0 otherwise, or a negative value if an error occurs.
+ */
+int
+xmlSecMSCngAppDefaultKeysMngrVerifyKey(xmlSecKeysMngrPtr mngr, xmlSecKeyPtr key, xmlSecKeyInfoCtxPtr keyInfoCtx) {
+
+    xmlSecAssert2(mngr != NULL, -1);
+    xmlSecAssert2(key != NULL, -1);
+    xmlSecAssert2(keyInfoCtx != NULL, -1);
+
+    xmlSecNotImplementedError("xmlSecMSCngAppDefaultKeysMngrVerifyKey");
+    return(-1);
+}
+
 /**
  * xmlSecMSCngAppDefaultKeysMngrLoad:
  * @mngr:               the pointer to keys manager.
@@ -823,7 +934,7 @@ xmlSecMSCngAppDefaultKeysMngrSave(xmlSecKeysMngrPtr mngr, const char* filename,
 
     ret = xmlSecMSCngKeysStoreSave(store, filename, type);
     if(ret < 0) {
-        xmlSecInternalError2("xmlSecMSCngKeysStoreSave", NULL, "filename%s",
+        xmlSecInternalError2("xmlSecMSCngKeysStoreSave", NULL, "filename=%s",
             xmlSecErrorsSafeString(filename));
         return(-1);
     }
@@ -843,4 +954,3 @@ xmlSecMSCngAppGetDefaultPwdCallback(void) {
     /* TODO */
     return(NULL);
 }
-
