@@ -1,17 +1,16 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
+ * RSA Key Transport transforms implementation for MSCng.
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  * Copyright (C) 2018 Miklos Vajna. All Rights Reserved.
  */
 /**
- * SECTION:kt_rsa
- * @Short_description: RSA Key Transport transforms implementation for Microsoft Cryptography API: Next Generation (CNG).
- * @Stability: Private
- *
+ * SECTION:crypto
  */
 
 #include "globals.h"
@@ -33,6 +32,7 @@
 #include <xmlsec/keyinfo.h>
 #include <xmlsec/private.h>
 #include <xmlsec/transforms.h>
+#include <xmlsec/xmltree.h>
 
 #include <xmlsec/mscng/crypto.h>
 #include <xmlsec/mscng/certkeys.h>
@@ -50,7 +50,7 @@ typedef struct _xmlSecMSCngRsaPkcs1OaepCtx xmlSecMSCngRsaPkcs1OaepCtx, *xmlSecMS
 struct _xmlSecMSCngRsaPkcs1OaepCtx {
     xmlSecKeyDataPtr data;
     xmlSecBuffer oaepParams;
-
+    LPCWSTR pszDigestAlgId;
 };
 
 /*********************************************************************
@@ -66,9 +66,9 @@ xmlSecMSCngRsaPkcs1OaepCheckId(xmlSecTransformPtr transform) {
 
     if(xmlSecTransformCheckId(transform, xmlSecMSCngTransformRsaPkcs1Id)) {
         return(1);
-    }
-
-    if(xmlSecTransformCheckId(transform, xmlSecMSCngTransformRsaOaepId)) {
+    } else if(xmlSecTransformCheckId(transform, xmlSecMSCngTransformRsaOaepId)) {
+        return(1);
+    } else if (xmlSecTransformCheckId(transform, xmlSecMSCngTransformRsaOaepEnc11Id)) {
         return(1);
     }
 
@@ -259,12 +259,11 @@ xmlSecMSCngRsaPkcs1OaepProcess(xmlSecTransformPtr transform) {
                     xmlSecTransformGetName(transform), status);
                 return(-1);
             }
-        } else if(xmlSecTransformCheckId(transform, xmlSecMSCngTransformRsaOaepId)) {
+        } else if(xmlSecTransformCheckId(transform, xmlSecMSCngTransformRsaOaepId) || xmlSecTransformCheckId(transform, xmlSecMSCngTransformRsaOaepEnc11Id)) {
             BCRYPT_OAEP_PADDING_INFO paddingInfo;
             xmlSecSize oaepParamsSize;
 
-
-            paddingInfo.pszAlgId = BCRYPT_SHA1_ALGORITHM;
+            paddingInfo.pszAlgId = ctx->pszDigestAlgId;
             paddingInfo.pbLabel = xmlSecBufferGetData(&(ctx->oaepParams));
 
             oaepParamsSize = xmlSecBufferGetSize(&(ctx->oaepParams));
@@ -319,11 +318,11 @@ xmlSecMSCngRsaPkcs1OaepProcess(xmlSecTransformPtr transform) {
                     xmlSecTransformGetName(transform), securityStatus);
                 return(-1);
             }
-        } else if(xmlSecTransformCheckId(transform, xmlSecMSCngTransformRsaOaepId)) {
+        } else if(xmlSecTransformCheckId(transform, xmlSecMSCngTransformRsaOaepId) || xmlSecTransformCheckId(transform, xmlSecMSCngTransformRsaOaepEnc11Id)) {
             BCRYPT_OAEP_PADDING_INFO paddingInfo;
             xmlSecSize oaepParamsSize;
 
-            paddingInfo.pszAlgId = BCRYPT_SHA1_ALGORITHM;
+            paddingInfo.pszAlgId = ctx->pszDigestAlgId;
             paddingInfo.pbLabel = xmlSecBufferGetData(&(ctx->oaepParams));
 
             oaepParamsSize = xmlSecBufferGetSize(&(ctx->oaepParams));
@@ -451,11 +450,13 @@ xmlSecMSCngTransformRsaPkcs1GetKlass(void) {
     return(&xmlSecMSCngRsaPkcs1Klass);
 }
 
+
 static int
 xmlSecMSCngRsaOaepNodeRead(xmlSecTransformPtr transform, xmlNodePtr node,
                            xmlSecTransformCtxPtr transformCtx ATTRIBUTE_UNUSED) {
     xmlSecMSCngRsaPkcs1OaepCtxPtr ctx;
-    xmlChar* algorithm = NULL;
+    xmlSecTransformRsaOaepParams oaepParams;
+    LPCWSTR mgf1AlgId = NULL;
     int ret;
 
     xmlSecAssert2(xmlSecMSCngRsaPkcs1OaepCheckId(transform), -1);
@@ -466,24 +467,127 @@ xmlSecMSCngRsaOaepNodeRead(xmlSecTransformPtr transform, xmlNodePtr node,
     ctx = xmlSecMSCngRsaPkcs1OaepGetCtx(transform);
     xmlSecAssert2(ctx != NULL, -1);
 
-    ret = xmlSecTransformRsaOaepReadParams(node, &(ctx->oaepParams), &algorithm);
+    ret = xmlSecTransformRsaOaepParamsInitialize(&oaepParams);
     if (ret < 0) {
-        xmlSecInternalError("xmlSecTransformRsaOaepReadParams",
+        xmlSecInternalError("xmlSecTransformRsaOaepParamsInitialize",
             xmlSecTransformGetName(transform));
         return(-1);
     }
 
-    /* for now we support only sha1 */
-    if ((algorithm != NULL) && (xmlStrcmp(algorithm, xmlSecHrefSha1) != 0)) {
-        xmlSecInvalidTransfromError2(transform,
-            "digest algorithm=\"%s\" is not supported for rsa/oaep",
-            xmlSecErrorsSafeString(algorithm));
-        xmlFree(algorithm);
+    ret = xmlSecTransformRsaOaepParamsRead(&oaepParams, node);
+    if (ret < 0) {
+        xmlSecInternalError("xmlSecTransformRsaOaepParamsRead",
+            xmlSecTransformGetName(transform));
+        xmlSecTransformRsaOaepParamsFinalize(&oaepParams);
         return(-1);
     }
-    xmlFree(algorithm);
 
-    /* done */
+    /* digest algorithm */
+    if (oaepParams.digestAlgorithm == NULL) {
+#ifndef XMLSEC_NO_SHA1
+        ctx->pszDigestAlgId = BCRYPT_SHA1_ALGORITHM;
+#else  /* XMLSEC_NO_SHA1 */
+        xmlSecOtherError(XMLSEC_ERRORS_R_DISABLED, NULL, "No OAEP digest algorithm is specified and the default SHA1 digest is disabled");
+        xmlSecTransformRsaOaepParamsFinalize(&oaepParams);
+        return(-1);
+#endif /* XMLSEC_NO_SHA1 */
+    } else
+#ifndef XMLSEC_NO_MD5
+    if (xmlStrcmp(oaepParams.digestAlgorithm, xmlSecHrefMd5) == 0) {
+        ctx->pszDigestAlgId = BCRYPT_MD5_ALGORITHM;
+    } else
+#endif /* XMLSEC_NO_MD5 */
+
+#ifndef XMLSEC_NO_SHA1
+    if (xmlStrcmp(oaepParams.digestAlgorithm, xmlSecHrefSha1) == 0) {
+        ctx->pszDigestAlgId = BCRYPT_SHA1_ALGORITHM;
+    } else
+#endif /* XMLSEC_NO_SHA1 */
+
+#ifndef XMLSEC_NO_SHA256
+    if (xmlStrcmp(oaepParams.digestAlgorithm, xmlSecHrefSha256) == 0) {
+        ctx->pszDigestAlgId = BCRYPT_SHA256_ALGORITHM;
+    } else
+#endif /* XMLSEC_NO_SHA256 */
+
+#ifndef XMLSEC_NO_SHA384
+    if (xmlStrcmp(oaepParams.digestAlgorithm, xmlSecHrefSha384) == 0) {
+        ctx->pszDigestAlgId = BCRYPT_SHA384_ALGORITHM;
+    } else
+#endif /* XMLSEC_NO_SHA384 */
+
+#ifndef XMLSEC_NO_SHA512
+    if (xmlStrcmp(oaepParams.digestAlgorithm, xmlSecHrefSha512) == 0) {
+        ctx->pszDigestAlgId = BCRYPT_SHA512_ALGORITHM;
+    } else
+#endif /* XMLSEC_NO_SHA512 */
+    {
+        xmlSecInvalidTransfromError2(transform,
+            "digest algorithm=\"%s\" is not supported for rsa/oaep",
+            xmlSecErrorsSafeString(oaepParams.digestAlgorithm));
+        xmlSecTransformRsaOaepParamsFinalize(&oaepParams);
+        return(-1);
+    }
+
+    /* mgf1 algorithm */
+    if (oaepParams.mgf1DigestAlgorithm == NULL) {
+#ifndef XMLSEC_NO_SHA1
+        mgf1AlgId = BCRYPT_SHA1_ALGORITHM;
+#else  /* XMLSEC_NO_SHA1 */
+        xmlSecOtherError(XMLSEC_ERRORS_R_DISABLED, NULL, "No OAEP mgf1 digest algorithm is specified and the default SHA1 digest is disabled");
+        xmlSecTransformRsaOaepParamsFinalize(&oaepParams);
+        return(-1);
+#endif /* XMLSEC_NO_SHA1 */
+    } else
+#ifndef XMLSEC_NO_SHA1
+    if (xmlStrcmp(oaepParams.mgf1DigestAlgorithm, xmlSecHrefMgf1Sha1) == 0) {
+        mgf1AlgId = BCRYPT_SHA1_ALGORITHM;
+    } else
+#endif /* XMLSEC_NO_SHA1 */
+
+#ifndef XMLSEC_NO_SHA256
+    if (xmlStrcmp(oaepParams.mgf1DigestAlgorithm, xmlSecHrefMgf1Sha256) == 0) {
+        mgf1AlgId = BCRYPT_SHA256_ALGORITHM;
+    } else
+#endif /* XMLSEC_NO_SHA256 */
+
+#ifndef XMLSEC_NO_SHA384
+    if (xmlStrcmp(oaepParams.mgf1DigestAlgorithm, xmlSecHrefMgf1Sha384) == 0) {
+        mgf1AlgId = BCRYPT_SHA384_ALGORITHM;
+    } else
+#endif /* XMLSEC_NO_SHA384 */
+
+#ifndef XMLSEC_NO_SHA512
+    if (xmlStrcmp(oaepParams.mgf1DigestAlgorithm, xmlSecHrefMgf1Sha512) == 0) {
+        mgf1AlgId = BCRYPT_SHA512_ALGORITHM;
+    } else
+#endif /* XMLSEC_NO_SHA512 */
+    {
+        xmlSecInvalidTransfromError2(transform,
+            "mgf1 digest algorithm=\"%s\" is not supported for rsa/oaep",
+            xmlSecErrorsSafeString(oaepParams.mgf1DigestAlgorithm));
+        xmlSecTransformRsaOaepParamsFinalize(&oaepParams);
+        return(-1);
+    }
+
+    /* MSCNG only supports *same* algorithms for digest and mgf1 */
+    if ((mgf1AlgId != NULL) && (ctx->pszDigestAlgId != NULL) && (lstrcmpW(ctx->pszDigestAlgId, mgf1AlgId) != 0)) {
+        xmlChar* digestAlg = xmlSecWin32ConvertUnicodeToUtf8(ctx->pszDigestAlgId);
+        xmlChar* mgf1Alg = xmlSecWin32ConvertUnicodeToUtf8(mgf1AlgId);
+
+        xmlSecInvalidTransfromError3(transform,
+            "for mscng, rsa/oaep mgf1 algorithm=\"%s\" must be the same as digest algorithm=\"%s\"",
+            xmlSecErrorsSafeString(digestAlg),
+            xmlSecErrorsSafeString(mgf1Alg));
+        xmlSecTransformRsaOaepParamsFinalize(&oaepParams);
+        return(-1);
+    }
+
+    /* put oaep params buffer into ctx */
+    xmlSecBufferSwap(&(oaepParams.oaepParams), &(ctx->oaepParams));
+
+    /* cleanup */
+    xmlSecTransformRsaOaepParamsFinalize(&oaepParams);
     return(0);
 }
 
@@ -517,7 +621,7 @@ static xmlSecTransformKlass xmlSecMSCngRsaOaepKlass = {
 /**
  * xmlSecMSCngTransformRsaOaepGetKlass:
  *
- * The RSA-OAEP key transport transform klass.
+ * The RSA-OAEP key transport transform klass (XMLEnc 1.0).
  *
  * Returns: RSA-OAEP key transport transform klass.
  */
@@ -526,4 +630,44 @@ xmlSecMSCngTransformRsaOaepGetKlass(void) {
     return(&xmlSecMSCngRsaOaepKlass);
 }
 
+
+static xmlSecTransformKlass xmlSecMSCngRsaOaepEnc11Klass = {
+    /* klass/object sizes */
+    sizeof(xmlSecTransformKlass),               /* xmlSecSize klassSize */
+    xmlSeccMSCngRsaPkcs1OaepSize,               /* xmlSecSize objSize */
+
+    xmlSecNameRsaOaepEnc11,                     /* const xmlChar* name; */
+    xmlSecHrefRsaOaepEnc11,                     /* const xmlChar* href; */
+    xmlSecTransformUsageEncryptionMethod,       /* xmlSecAlgorithmUsage usage; */
+
+    xmlSecMSCngRsaPkcs1OaepInitialize,          /* xmlSecTransformInitializeMethod initialize; */
+    xmlSecMSCngRsaPkcs1OaepFinalize,            /* xmlSecTransformFinalizeMethod finalize; */
+    xmlSecMSCngRsaOaepNodeRead,                 /* xmlSecTransformNodeReadMethod readNode; */
+    NULL,                                       /* xmlSecTransformNodeWriteMethod writeNode; */
+    xmlSecMSCngRsaPkcs1OaepSetKeyReq,           /* xmlSecTransformSetKeyMethod setKeyReq; */
+    xmlSecMSCngRsaPkcs1OaepSetKey,              /* xmlSecTransformSetKeyMethod setKey; */
+    NULL,                                       /* xmlSecTransformValidateMethod validate; */
+    xmlSecTransformDefaultGetDataType,          /* xmlSecTransformGetDataTypeMethod getDataType; */
+    xmlSecTransformDefaultPushBin,              /* xmlSecTransformPushBinMethod pushBin; */
+    xmlSecTransformDefaultPopBin,               /* xmlSecTransformPopBinMethod popBin; */
+    NULL,                                       /* xmlSecTransformPushXmlMethod pushXml; */
+    NULL,                                       /* xmlSecTransformPopXmlMethod popXml; */
+    xmlSecMSCngRsaPkcs1OaepExecute,             /* xmlSecTransformExecuteMethod execute; */
+
+    NULL,                                       /* void* reserved0; */
+    NULL,                                       /* void* reserved1; */
+};
+
+/**
+ * xmlSecMSCngTransformRsaOaepEnc11GetKlass:
+ *
+ * The RSA-OAEP key transport transform klass (XMLEnc 1.1).
+ *
+ * Returns: RSA-OAEP key transport transform klass.
+ */
+xmlSecTransformId
+xmlSecMSCngTransformRsaOaepEnc11GetKlass(void) {
+    return(&xmlSecMSCngRsaOaepEnc11Klass);
+}
+
 #endif /* XMLSEC_NO_RSA */
