@@ -1,17 +1,20 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
+ * X509 certificates implementation for MSCng.
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
+* Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  * Copyright (C) 2018 Miklos Vajna. All Rights Reserved.
  */
 /**
  * SECTION:x509
- * @Short_description: X509 certificates implementation for Microsoft Cryptography API: Next Generation (CNG).
+ * @Short_description: X509 certificates implementation for MSCng.
  * @Stability: Stable
  *
+ * X509 certificates implementation for MSCng.
  */
 
 #include "globals.h"
@@ -30,6 +33,7 @@
 #include <xmlsec/keyinfo.h>
 #include <xmlsec/keysmngr.h>
 #include <xmlsec/xmltree.h>
+#include <xmlsec/private.h>
 #include <xmlsec/x509.h>
 
 #include <xmlsec/mscng/certkeys.h>
@@ -38,13 +42,14 @@
 
 #include "../cast_helpers.h"
 #include "../keysdata_helpers.h"
+#include "private.h"
 
 typedef struct _xmlSecMSCngX509DataCtx xmlSecMSCngX509DataCtx,
                                        *xmlSecMSCngX509DataCtxPtr;
 
 struct _xmlSecMSCngX509DataCtx {
     HCERTSTORE hMemStore;
-    PCCERT_CONTEXT cert;
+    PCCERT_CONTEXT keyCert; /* owned by hMemStore */
 };
 
 XMLSEC_KEY_DATA_DECLARE(MSCngX509Data, xmlSecMSCngX509DataCtx)
@@ -72,56 +77,6 @@ xmlSecMSCngKeyDataX509Initialize(xmlSecKeyDataPtr data) {
     return(0);
 }
 
-static int
-xmlSecMSCngKeyDataX509Duplicate(xmlSecKeyDataPtr dst, xmlSecKeyDataPtr src) {
-    PCCERT_CONTEXT srcCert = NULL;
-    PCCERT_CONTEXT dstCert;
-    xmlSecMSCngX509DataCtxPtr srcCtx;
-    int ret;
-
-    xmlSecAssert2(xmlSecKeyDataCheckId(dst, xmlSecMSCngKeyDataX509Id), -1);
-    xmlSecAssert2(xmlSecKeyDataCheckId(src, xmlSecMSCngKeyDataX509Id), -1);
-    srcCtx = xmlSecMSCngX509DataGetCtx(src);
-
-    /* duplicate the certificate store */
-    while((srcCert = CertEnumCertificatesInStore(srcCtx->hMemStore, srcCert)) != NULL) {
-        dstCert = CertDuplicateCertificateContext(srcCert);
-        if(dstCert == NULL) {
-            xmlSecMSCngLastError("CertDuplicateCertificateContext",
-                xmlSecKeyDataGetName(dst));
-            return(-1);
-        }
-
-        ret = xmlSecMSCngKeyDataX509AdoptCert(dst, dstCert);
-        if(ret < 0) {
-            xmlSecInternalError("xmlSecMSCngKeyDataX509AdoptCert",
-                xmlSecKeyDataGetName(dst));
-            CertFreeCertificateContext(dstCert);
-            return(-1);
-        }
-    }
-
-    if(srcCtx->cert != NULL) {
-        /* have a key certificate, duplicate that */
-        dstCert = CertDuplicateCertificateContext(srcCtx->cert);
-        if(dstCert == NULL) {
-            xmlSecMSCngLastError("CertDuplicateCertificateContext",
-                xmlSecKeyDataGetName(dst));
-            return(-1);
-        }
-
-        ret = xmlSecMSCngKeyDataX509AdoptKeyCert(dst, dstCert);
-        if(ret < 0) {
-            xmlSecInternalError("xmlSecMSCngKeyDataX509AdoptKeyCert",
-                xmlSecKeyDataGetName(dst));
-            CertFreeCertificateContext(dstCert);
-            return(-1);
-        }
-    }
-
-    return(0);
-}
-
 static void
 xmlSecMSCngKeyDataX509Finalize(xmlSecKeyDataPtr data) {
     xmlSecMSCngX509DataCtxPtr ctx;
@@ -131,15 +86,8 @@ xmlSecMSCngKeyDataX509Finalize(xmlSecKeyDataPtr data) {
     ctx = xmlSecMSCngX509DataGetCtx(data);
     xmlSecAssert(ctx != NULL);
 
-    if(ctx->cert != NULL) {
-        if(!CertFreeCertificateContext(ctx->cert)) {
-            xmlSecMSCngLastError("CertFreeCertificateContext", NULL);
-            /* ignore error */
-        }
-    }
-
-    if(ctx->hMemStore != 0) {
-        if(!CertCloseStore(ctx->hMemStore, 0)) {
+    if (ctx->hMemStore != 0) {
+        if (!CertCloseStore(ctx->hMemStore, 0)) {
             xmlSecMSCngLastError("CertCloseStore", NULL);
             /* ignore error */
         }
@@ -148,6 +96,54 @@ xmlSecMSCngKeyDataX509Finalize(xmlSecKeyDataPtr data) {
     memset(ctx, 0, sizeof(xmlSecMSCngX509DataCtx));
 }
 
+static int
+xmlSecMSCngKeyDataX509Duplicate(xmlSecKeyDataPtr dst, xmlSecKeyDataPtr src) {
+    PCCERT_CONTEXT srcCert = NULL;
+    PCCERT_CONTEXT dstCert;
+    xmlSecMSCngX509DataCtxPtr srcCtx;
+    xmlSecMSCngX509DataCtxPtr dstCtx;
+    int ret;
+
+    xmlSecAssert2(xmlSecKeyDataCheckId(dst, xmlSecMSCngKeyDataX509Id), -1);
+    xmlSecAssert2(xmlSecKeyDataCheckId(src, xmlSecMSCngKeyDataX509Id), -1);
+
+    srcCtx = xmlSecMSCngX509DataGetCtx(src);
+    xmlSecAssert2(srcCtx != NULL, -1);
+    dstCtx = xmlSecMSCngX509DataGetCtx(dst);
+    xmlSecAssert2(dstCtx != NULL, -1);
+
+    /* duplicate the certificate store */
+    while((srcCert = CertEnumCertificatesInStore(srcCtx->hMemStore, srcCert)) != NULL) {
+        dstCert = CertDuplicateCertificateContext(srcCert);
+        if(dstCert == NULL) {
+            xmlSecMSCngLastError("CertDuplicateCertificateContext", NULL);
+            return(-1);
+        }
+
+        /* ensure to handle keyCert */
+        if (srcCert == srcCtx->keyCert) {
+            ret = xmlSecMSCngKeyDataX509AdoptKeyCert(dst, dstCert);
+            if (ret < 0) {
+                xmlSecInternalError("xmlSecMSCngKeyDataX509AdoptKeyCert", NULL);
+                CertFreeCertificateContext(dstCert);
+                return(-1);
+            }
+        } else {
+            ret = xmlSecMSCngKeyDataX509AdoptCert(dst, dstCert);
+            if (ret < 0) {
+                xmlSecInternalError("xmlSecMSCngKeyDataX509AdoptCert", NULL);
+                CertFreeCertificateContext(dstCert);
+                return(-1);
+            }
+        }
+        dstCert = NULL; /* owned by dst now */
+    }
+
+    /* done */
+    return(0);
+}
+
+
 static xmlSecKeyDataType
 xmlSecMSCngKeyDataX509GetType(xmlSecKeyDataPtr data) {
     xmlSecAssert2(xmlSecKeyDataCheckId(data, xmlSecMSCngKeyDataX509Id), xmlSecKeyDataTypeUnknown);
@@ -184,10 +180,39 @@ xmlSecMSCngX509CertDerRead(const xmlSecByte* buf, xmlSecSize size) {
 
     return(cert);
 }
+static int
+xmlSecMSCngKeyDataX509AddCertInternal(xmlSecMSCngX509DataCtxPtr ctx, PCCERT_CONTEXT cert) {
+    xmlSecAssert2(ctx != NULL, -1);
+    xmlSecAssert2(cert != NULL, -1);
 
+    if (!CertAddCertificateContextToStore(ctx->hMemStore,
+        cert,
+        CERT_STORE_ADD_ALWAYS,
+        NULL
+    )) {
+        xmlSecMSCngLastError("CertAddCertificateContextToStore", NULL);
+        return(-1);
+    }
+
+    /* caller expects data to own the cert on success. */
+    CertFreeCertificateContext(cert);
+    return(0);
+}
+
+/**
+ * xmlSecMSCngKeyDataX509AdoptKeyCert:
+ * @data:    the pointer to key data.
+ * @cert:    the pointer to certificates.
+ *
+ * Adds certificate to the X509 key data and sets the it as the key's
+ * certificate in @data. On success, the @data owns the cert.
+ *
+ * Returns: 0 on success or a negative value otherwise.
+ */
 int
 xmlSecMSCngKeyDataX509AdoptKeyCert(xmlSecKeyDataPtr data, PCCERT_CONTEXT cert) {
     xmlSecMSCngX509DataCtxPtr ctx;
+    int ret;
 
     xmlSecAssert2(xmlSecKeyDataCheckId(data, xmlSecMSCngKeyDataX509Id), -1);
     xmlSecAssert2(cert != NULL, -1);
@@ -195,14 +220,33 @@ xmlSecMSCngKeyDataX509AdoptKeyCert(xmlSecKeyDataPtr data, PCCERT_CONTEXT cert) {
     ctx = xmlSecMSCngX509DataGetCtx(data);
     xmlSecAssert2(ctx != NULL, -1);
 
-    if(ctx->cert != NULL) {
-        CertFreeCertificateContext(ctx->cert);
+    /* check if for some reasons same cert is used */
+    if ((ctx->keyCert != NULL) && (CertCompareCertificate(X509_ASN_ENCODING | PKCS_7_ASN_ENCODING, cert->pCertInfo, ctx->keyCert->pCertInfo) == TRUE)) {
+        CertFreeCertificateContext(cert);  /* caller expects data to own the cert on success. */
+        return(0);
     }
-    ctx->cert = cert;
+    xmlSecAssert2(ctx->keyCert == NULL, -1);
 
+    ret = xmlSecMSCngKeyDataX509AddCertInternal(ctx, cert);
+    if (ret < 0) {
+        xmlSecInternalError("xmlSecMSCngKeyDataX509AddCertInternal", xmlSecKeyDataGetName(data));
+        return(-1);
+    }
+
+    /* cert is now owned by data, we can't fail or there will be a double free */
+    ctx->keyCert = cert;
     return(0);
 }
 
+/**
+ * xmlSecMSCngKeyDataX509AdoptCert:
+ * @data:    the pointer to key data.
+ * @cert:    the pointer to certificates.
+ *
+ * Adds @cert to @data as a certificate. On success, @data owns the @cert.
+ *
+ * Returns: 0 on success or a negative value otherwise.
+ */
 int
 xmlSecMSCngKeyDataX509AdoptCert(xmlSecKeyDataPtr data, PCCERT_CONTEXT cert) {
     xmlSecMSCngX509DataCtxPtr ctx;
@@ -214,18 +258,12 @@ xmlSecMSCngKeyDataX509AdoptCert(xmlSecKeyDataPtr data, PCCERT_CONTEXT cert) {
     xmlSecAssert2(ctx != NULL, -1);
     xmlSecAssert2(ctx->hMemStore != 0, -1);
 
-    if(!CertAddCertificateContextToStore(ctx->hMemStore,
-        cert,
-        CERT_STORE_ADD_ALWAYS,
-        NULL)) {
-        xmlSecMSCngLastError("CertAddCertificateContextToStore",
-            xmlSecKeyDataGetName(data));
-        return(-1);
+    /* pkcs12 files sometime have key cert twice: as the key cert and as the cert in the chain */
+    if ((ctx->keyCert != NULL) && (CertCompareCertificate(X509_ASN_ENCODING | PKCS_7_ASN_ENCODING, cert->pCertInfo, ctx->keyCert->pCertInfo) == TRUE)) {
+        CertFreeCertificateContext(cert); /* caller expects data to own the cert on success. */
+        return(0);
     }
-
-    /* this just decrements the refcount, so won't free */
-    CertFreeCertificateContext(cert);
-    return(0);
+    return(xmlSecMSCngKeyDataX509AddCertInternal(ctx, cert));
 }
 
 /**
@@ -249,8 +287,7 @@ xmlSecMSCngKeyDataX509AdoptCrl(xmlSecKeyDataPtr data, PCCRL_CONTEXT crl) {
     xmlSecAssert2(ctx->hMemStore != 0, -1);
 
     if (!CertAddCRLContextToStore(ctx->hMemStore, crl, CERT_STORE_ADD_ALWAYS, NULL)) {
-        xmlSecMSCngLastError("CertAddCRLContextToStore",
-            xmlSecKeyDataGetName(data));
+        xmlSecMSCngLastError("CertAddCRLContextToStore", NULL);
         return(-1);
     }
 
@@ -298,121 +335,134 @@ xmlSecMSCngX509CertGetTime(FILETIME in, time_t* out) {
     return(0);
 }
 
+/* returns 1 if cert was found and verified and also data was adopted, 0 if not, or negative value if an error occurs */
 static int
-xmlSecMSCngKeyDataX509VerifyAndExtractKey(xmlSecKeyDataPtr data,
-    xmlSecKeyPtr key, xmlSecKeyInfoCtxPtr keyInfoCtx) {
+xmlSecMSCnVerifyAndAdoptX509KeyData(xmlSecKeyPtr key, xmlSecKeyDataPtr data, xmlSecKeyInfoCtxPtr keyInfoCtx) {
     xmlSecMSCngX509DataCtxPtr ctx;
-    xmlSecKeyDataStorePtr store;
+    xmlSecKeyDataStorePtr x509Store;
+    xmlSecKeyDataPtr keyValue;
     PCCERT_CONTEXT cert;
+    PCCERT_CONTEXT certCopy;
+    PCCERT_CONTEXT keyCert;
+    int ret;
 
     xmlSecAssert2(xmlSecKeyDataCheckId(data, xmlSecMSCngKeyDataX509Id), -1);
     xmlSecAssert2(key != NULL, -1);
     xmlSecAssert2(keyInfoCtx != NULL, -1);
     xmlSecAssert2(keyInfoCtx->keysMngr != NULL, -1);
 
-    if(xmlSecKeyGetValue(key) != NULL) {
-        return(0);
-    }
-
     ctx = xmlSecMSCngX509DataGetCtx(data);
     xmlSecAssert2(ctx != NULL, -1);
     xmlSecAssert2(ctx->hMemStore != 0, -1);
+    xmlSecAssert2(ctx->keyCert == NULL, -1);
 
-    if(ctx->cert != NULL) {
+
+    if (xmlSecKeyGetValue(key) != NULL) {
+        /* key was already found -> nothing to do (this shouldn't really happen) */
         return(0);
     }
 
-    store = xmlSecKeysMngrGetDataStore(keyInfoCtx->keysMngr, xmlSecMSCngX509StoreId);
-    if(store == NULL) {
-        xmlSecInternalError("xmlSecKeysMngrGetDataStore",
-            xmlSecKeyDataGetName(data));
+    /* lets find a cert we can verify */
+    x509Store = xmlSecKeysMngrGetDataStore(keyInfoCtx->keysMngr, xmlSecMSCngX509StoreId);
+    if(x509Store == NULL) {
+        xmlSecInternalError("xmlSecKeysMngrGetDataStore", xmlSecKeyDataGetName(data));
+        return(-1);
+    }
+    cert = xmlSecMSCngX509StoreVerify(x509Store, ctx->hMemStore, keyInfoCtx);
+    if (cert == NULL) {
+        /* check if we want to fail if cert is not found */
+        if ((keyInfoCtx->flags & XMLSEC_KEYINFO_FLAGS_X509DATA_STOP_ON_INVALID_CERT) != 0) {
+            xmlSecOtherError(XMLSEC_ERRORS_R_CERT_NOT_FOUND, xmlSecKeyDataGetName(data), NULL);
+            return(-1);
+        }
+        return(0);
+    }
+
+    /* set cert into the x509 data, we don't know if the cert is already in KeyData or not
+     * so assume we need to add it again.
+     */
+    keyCert = CertDuplicateCertificateContext(cert);
+    if(keyCert == NULL) {
+        xmlSecMSCngLastError("CertDuplicateCertificateContext", xmlSecKeyDataGetName(data));
+        return(-1);
+    }
+    ret = xmlSecMSCngKeyDataX509AdoptKeyCert(data, keyCert);
+    if (ret < 0) {
+        xmlSecInternalError("xmlSecMSCngKeyDataX509AdoptKeyCert", xmlSecKeyDataGetName(data));
+        CertFreeCertificateContext(keyCert);
+        return(-1);
+    }
+    cert = keyCert = NULL; /* we should be using ctx->keyCert for everything */
+
+    /* extract key from cert (need to copy the certificate, so it can be adopted according to the key value data) */
+    certCopy = CertDuplicateCertificateContext(ctx->keyCert);
+    if(certCopy == NULL) {
+        xmlSecMSCngLastError("CertDuplicateCertificateContext", xmlSecKeyDataGetName(data));
+        return(-1);
+    }
+    if((keyInfoCtx->keyReq.keyType & xmlSecKeyDataTypePrivate) != 0) {
+        keyValue = xmlSecMSCngCertAdopt(certCopy, xmlSecKeyDataTypePrivate);
+        if(keyValue == NULL) {
+            xmlSecInternalError("xmlSecMSCngCertAdopt", xmlSecKeyDataGetName(data));
+            CertFreeCertificateContext(certCopy);
+            return(-1);
+        }
+    } else {
+        /* assume we want a public key (if we don't want private) */
+        keyValue = xmlSecMSCngCertAdopt(certCopy, xmlSecKeyDataTypePublic);
+        if(keyValue == NULL) {
+            xmlSecInternalError("xmlSecMSCngCertAdopt", xmlSecKeyDataGetName(data));
+            CertFreeCertificateContext(certCopy);
+            return(-1);
+        }
+    }
+    certCopy = NULL; /* owned by key value now */
+
+    /* verify that keyValue matches the key requirements */
+    if(xmlSecKeyReqMatchKeyValue(&(keyInfoCtx->keyReq), keyValue) != 1) {
+        xmlSecInternalError("xmlSecKeyReqMatchKeyValue", xmlSecKeyDataGetName(data));
+        xmlSecKeyDataDestroy(keyValue);
         return(-1);
     }
 
-    cert = xmlSecMSCngX509StoreVerify(store, ctx->hMemStore, keyInfoCtx);
-    if(cert != NULL) {
-        int ret;
-        PCCERT_CONTEXT certCopy;
-        xmlSecKeyDataPtr keyValue = NULL;
+    /* set key value */
+    ret = xmlSecKeySetValue(key, keyValue);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecKeySetValue", xmlSecKeyDataGetName(data));
+        xmlSecKeyDataDestroy(keyValue);
+        return(-1);
+    }
+    keyValue = NULL; /* owned by key now */
 
-        ctx->cert = CertDuplicateCertificateContext(cert);
-        if(ctx->cert == NULL) {
-            xmlSecMSCngLastError("CertDuplicateCertificateContext",
-                xmlSecKeyDataGetName(data));
-            return(-1);
-        }
-
-        /* copy the certificate, so it can be adopted according to the key data
-         * type */
-        certCopy = CertDuplicateCertificateContext(ctx->cert);
-        if(certCopy == NULL) {
-            xmlSecMSCngLastError("CertDuplicateCertificateContext",
-                xmlSecKeyDataGetName(data));
-            return(-1);
-        }
-
-        if((keyInfoCtx->keyReq.keyType & xmlSecKeyDataTypePrivate) != 0) {
-            keyValue = xmlSecMSCngCertAdopt(certCopy, xmlSecKeyDataTypePrivate);
-            if(keyValue == NULL) {
-                xmlSecInternalError("xmlSecMSCngCertAdopt",
-                    xmlSecKeyDataGetName(data));
-                return(-1);
-            }
-        } else if((keyInfoCtx->keyReq.keyType & xmlSecKeyDataTypePublic) != 0) {
-            keyValue = xmlSecMSCngCertAdopt(certCopy, xmlSecKeyDataTypePublic);
-            if(keyValue == NULL) {
-                xmlSecInternalError("xmlSecMSCngCertAdopt",
-                    xmlSecKeyDataGetName(data));
-                return(-1);
-            }
-        }
-
-        /* verify that keyValue matches the key requirements */
-        if(xmlSecKeyReqMatchKeyValue(&(keyInfoCtx->keyReq), keyValue) != 1) {
-            xmlSecInternalError("xmlSecKeyReqMatchKeyValue",
-                xmlSecKeyDataGetName(data));
-            xmlSecKeyDataDestroy(keyValue);
-            return(-1);
-        }
-
-        ret = xmlSecKeySetValue(key, keyValue);
-        if(ret < 0) {
-            xmlSecInternalError("xmlSecKeySetValue",
-                xmlSecKeyDataGetName(data));
-            xmlSecKeyDataDestroy(keyValue);
-            return(-1);
-        }
-
-        ret = xmlSecMSCngX509CertGetTime(ctx->cert->pCertInfo->NotBefore,
-            &(key->notValidBefore));
-        if(ret < 0) {
-            xmlSecInternalError("xmlSecMSCngX509CertGetTime",
-                xmlSecKeyDataGetName(data));
-            return(-1);
-        }
-
-        ret = xmlSecMSCngX509CertGetTime(ctx->cert->pCertInfo->NotAfter,
-            &(key->notValidAfter));
-        if(ret < 0) {
-            xmlSecInternalError("xmlSecMSCngX509CertGetTime",
-                xmlSecKeyDataGetName(data));
-            return(-1);
-        }
-    } else if((keyInfoCtx->flags &
-            XMLSEC_KEYINFO_FLAGS_X509DATA_STOP_ON_INVALID_CERT) != 0) {
-        xmlSecOtherError(XMLSEC_ERRORS_R_CERT_NOT_FOUND,
-            xmlSecKeyDataGetName(data), NULL);
+    /* copy cert not before / not after times from the cert */
+    ret = xmlSecMSCngX509CertGetTime(ctx->keyCert->pCertInfo->NotBefore, &(key->notValidBefore));
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecMSCngX509CertGetTime", xmlSecKeyDataGetName(data));
+        return(-1);
+    }
+    ret = xmlSecMSCngX509CertGetTime(ctx->keyCert->pCertInfo->NotAfter, &(key->notValidAfter));
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecMSCngX509CertGetTime", xmlSecKeyDataGetName(data));
         return(-1);
     }
 
-    return(0);
+    /* THIS MUST BE THE LAST THING WE DO: add data to the key
+     * if we do it sooner and fail later then both the caller and the key will free data
+     * which would lead to double free */
+    ret = xmlSecKeyAdoptData(key, data);
+    if (ret < 0) {
+        xmlSecInternalError("xmlSecKeyAdoptData", xmlSecKeyDataGetName(data));
+        return(-1);
+    }
+
+    /* success: cert found and data was adopted */
+    return(1);
 }
 
+/* xmlSecKeyDataX509Read: 0 on success and a negative value otherwise */
 static int
-xmlSecMSCngKeyDataX509Read(xmlSecKeyDataPtr data, xmlSecKeyValueX509Ptr x509Value,
+xmlSecMSCngKeyDataX509Read(xmlSecKeyDataPtr data, xmlSecKeyX509DataValuePtr x509Value,
     xmlSecKeysMngrPtr keysMngr, unsigned int flags) {
-    xmlSecKeyDataStorePtr x509Store;
-    int stopOnUnknownCert = 0;
     PCCERT_CONTEXT cert = NULL;
     PCCRL_CONTEXT crl = NULL;
     int ret;
@@ -423,17 +473,7 @@ xmlSecMSCngKeyDataX509Read(xmlSecKeyDataPtr data, xmlSecKeyValueX509Ptr x509Valu
     xmlSecAssert2(x509Value != NULL, -1);
     xmlSecAssert2(keysMngr != NULL, -1);
 
-    x509Store = xmlSecKeysMngrGetDataStore(keysMngr, xmlSecMSCngX509StoreId);
-    if (x509Store == NULL) {
-        xmlSecInternalError("xmlSecKeysMngrGetDataStore", xmlSecKeyDataGetName(data));
-        goto done;
-    }
-
-    /* determine what to do */
-    if ((flags & XMLSEC_KEYINFO_FLAGS_X509DATA_STOP_ON_UNKNOWN_CERT) != 0) {
-        stopOnUnknownCert = 1;
-    }
-
+    /* read CRT or CRL */
     if (xmlSecBufferGetSize(&(x509Value->cert)) > 0) {
         cert = xmlSecMSCngX509CertDerRead(xmlSecBufferGetData(&(x509Value->cert)),
             xmlSecBufferGetSize(&(x509Value->cert)));
@@ -442,7 +482,7 @@ xmlSecMSCngKeyDataX509Read(xmlSecKeyDataPtr data, xmlSecKeyValueX509Ptr x509Valu
             goto done;
         }
     }
-    else if (xmlSecBufferGetSize(&(x509Value->crl)) > 0) {
+    if (xmlSecBufferGetSize(&(x509Value->crl)) > 0) {
         crl = xmlSecMSCngX509CrlDerRead(xmlSecBufferGetData(&(x509Value->crl)),
             xmlSecBufferGetSize(&(x509Value->crl)));
         if (crl == NULL) {
@@ -450,34 +490,25 @@ xmlSecMSCngKeyDataX509Read(xmlSecKeyDataPtr data, xmlSecKeyValueX509Ptr x509Valu
             goto done;
         }
     }
-    else if (xmlSecBufferGetSize(&(x509Value->ski)) > 0) {
-        cert = xmlSecMSCngX509StoreFindCert_ex(x509Store, NULL, NULL, NULL,
-            xmlSecBufferGetData(&(x509Value->ski)), xmlSecBufferGetSize(&(x509Value->ski)),
-            NULL /* unused */);
-        if ((cert == NULL) && (stopOnUnknownCert != 0)) {
-            xmlSecOtherError2(XMLSEC_ERRORS_R_CERT_NOT_FOUND, xmlSecKeyDataGetName(data),
-                "skiSize=" XMLSEC_SIZE_FMT, xmlSecBufferGetSize(&(x509Value->ski)));
+
+    /* if there is no cert in the X509Data node then try to find one */
+    if (cert == NULL) {
+        xmlSecKeyDataStorePtr x509Store;
+        int stopOnUnknownCert = 0;
+
+        x509Store = xmlSecKeysMngrGetDataStore(keysMngr, xmlSecMSCngX509StoreId);
+        if (x509Store == NULL) {
+            xmlSecInternalError("xmlSecKeysMngrGetDataStore", xmlSecKeyDataGetName(data));
             goto done;
         }
-    }
-    else if (x509Value->subject != NULL) {
-        cert = xmlSecMSCngX509StoreFindCert_ex(x509Store, x509Value->subject,
-            NULL, NULL, NULL, 0, NULL /* unused */);
-        if ((cert == NULL) && (stopOnUnknownCert != 0)) {
-            xmlSecOtherError2(XMLSEC_ERRORS_R_CERT_NOT_FOUND, xmlSecKeyDataGetName(data),
-                "subject=%s", xmlSecErrorsSafeString(x509Value->subject));
-            goto done;
+        /* determine what to do */
+        if ((flags & XMLSEC_KEYINFO_FLAGS_X509DATA_STOP_ON_UNKNOWN_CERT) != 0) {
+            stopOnUnknownCert = 1;
         }
-    }
-    else if ((x509Value->issuerName != NULL) && (x509Value->issuerSerial != NULL)) {
-        cert = xmlSecMSCngX509StoreFindCert_ex(x509Store, NULL,
-            x509Value->issuerName, x509Value->issuerSerial,
-            NULL, 0, NULL /* unused */);
+
+        cert = xmlSecMSCngX509StoreFindCertByValue(x509Store, x509Value);
         if ((cert == NULL) && (stopOnUnknownCert != 0)) {
-            xmlSecOtherError3(XMLSEC_ERRORS_R_CERT_NOT_FOUND, xmlSecKeyDataGetName(data),
-                "issuerName=%s;issuerSerial=%s",
-                xmlSecErrorsSafeString(x509Value->issuerName),
-                xmlSecErrorsSafeString(x509Value->issuerSerial));
+            xmlSecOtherError(XMLSEC_ERRORS_R_CERT_NOT_FOUND, xmlSecKeyDataGetName(data), "cert lookup");
             goto done;
         }
     }
@@ -523,27 +554,37 @@ xmlSecMSCngKeyDataX509XmlRead(xmlSecKeyDataId id, xmlSecKeyPtr key,
     xmlSecAssert2(id == xmlSecMSCngKeyDataX509Id, -1);
     xmlSecAssert2(key != NULL, -1);
 
-    data = xmlSecKeyEnsureData(key, id);
+    data = xmlSecKeyDataCreate(xmlSecMSCngKeyDataX509Id);
     if (data == NULL) {
-        xmlSecInternalError("xmlSecKeyEnsureData",
-            xmlSecKeyDataKlassGetName(id));
+        xmlSecInternalError("xmlSecKeyDataCreate(xmlSecMSCngKeyDataX509Id)", xmlSecKeyDataKlassGetName(id));
         return(-1);
     }
 
-    ret = xmlSecKeyDataX509XmlRead(data, node, keyInfoCtx,
-        xmlSecMSCngKeyDataX509Read);
+    ret = xmlSecKeyDataX509XmlRead(key, data, node, keyInfoCtx, xmlSecMSCngKeyDataX509Read);
     if (ret < 0) {
-        xmlSecInternalError("xmlSecKeyDataX509XmlRead",
-            xmlSecKeyDataKlassGetName(id));
+        xmlSecInternalError("xmlSecKeyDataX509XmlRead", xmlSecKeyDataKlassGetName(id));
+        xmlSecKeyDataDestroy(data);
         return(-1);
     }
 
-    ret = xmlSecMSCngKeyDataX509VerifyAndExtractKey(data, key, keyInfoCtx);
-    if (ret < 0) {
-        xmlSecInternalError("xmlSecMSCngKeyDataX509VerifyAndExtractKey",
-            xmlSecKeyDataKlassGetName(id));
-        return(-1);
+    /* did we find the key already? */
+    if (xmlSecKeyGetValue(key) != NULL) {
+        xmlSecKeyDataDestroy(data);
+        return(0);
     }
+
+    ret = xmlSecMSCnVerifyAndAdoptX509KeyData(key, data, keyInfoCtx);
+    if (ret < 0) {
+        xmlSecInternalError("xmlSecMSCnVerifyAndAdoptX509KeyData", xmlSecKeyDataKlassGetName(id));
+        xmlSecKeyDataDestroy(data);
+    } else if (ret != 1) {
+        /* no errors but key was not found and data was not adopted */
+        xmlSecKeyDataDestroy(data);
+        return(0);
+    }
+    data = NULL; /* owned by data now */
+
+    /* success */
     return(0);
 }
 
@@ -673,6 +714,45 @@ xmlSecMSCngX509SKIWrite(PCCERT_CONTEXT cert, xmlSecBufferPtr buf) {
     return(0);
 }
 
+#define XMLSEC_MSCNG_SHA1_DIGEST_SIZE 20
+
+static int
+xmlSecMSCngX509DigestWrite(PCCERT_CONTEXT cert, const xmlChar* algorithm, xmlSecBufferPtr buf) {
+    xmlSecByte md[XMLSEC_MSCNG_SHA1_DIGEST_SIZE];
+    DWORD mdLen = sizeof(md);
+    BOOL status;
+    int ret;
+
+    xmlSecAssert2(cert != NULL, -1);
+    xmlSecAssert2(buf != NULL, -1);
+
+    /* only SHA1 algorithm is currently supported */
+    if (xmlStrcmp(algorithm, xmlSecHrefSha1) != 0) {
+        xmlSecOtherError2(XMLSEC_ERRORS_R_INVALID_ALGORITHM, NULL,
+            "href=%s", xmlSecErrorsSafeString(algorithm));
+        return(-1);
+    }
+
+    status = CertGetCertificateContextProperty(cert,
+        CERT_SHA1_HASH_PROP_ID,
+        md,
+        &mdLen);
+    if ((!status) || (mdLen != sizeof(md))) {
+        xmlSecMSCngLastError("CertGetCertificateContextProperty", NULL);
+        return(-1);
+    }
+
+    ret = xmlSecBufferSetData(buf, md, mdLen);
+    if (ret < 0) {
+        xmlSecInternalError("xmlSecBufferSetData", NULL);
+        return(-1);
+    }
+
+    /* success */
+    return(0);
+}
+
+
 typedef struct _xmlSecMSCngKeyDataX5099WriteContext {
     HCERTSTORE store;
     PCCERT_CONTEXT crt;
@@ -681,8 +761,11 @@ typedef struct _xmlSecMSCngKeyDataX5099WriteContext {
     int doneCrls;
 } xmlSecMSCngKeyDataX5099WriteContext;
 
+/* xmlSecKeyDataX509Write: returns 1 on success, 0 if no more certs/crls are available,
+ * or a negative value if an error occurs.
+ */
 static int
-xmlSecMSCngKeyDataX509Write(xmlSecKeyDataPtr data, xmlSecKeyValueX509Ptr x509Value,
+xmlSecMSCngKeyDataX509Write(xmlSecKeyDataPtr data, xmlSecKeyX509DataValuePtr x509Value,
                             int content, void* context) {
     xmlSecMSCngKeyDataX5099WriteContext* ctx;
     int ret;
@@ -700,7 +783,7 @@ xmlSecMSCngKeyDataX509Write(xmlSecKeyDataPtr data, xmlSecKeyValueX509Ptr x509Val
     if (ctx->doneCrts == 0) {
         ctx->crt = CertEnumCertificatesInStore(ctx->store, ctx->crt);
         if (ctx->crt != NULL) {
-            if ((content & XMLSEC_X509DATA_CERTIFICATE_NODE) != 0) {
+            if (XMLSEC_X509DATA_HAS_EMPTY_NODE(content, XMLSEC_X509DATA_CERTIFICATE_NODE)) {
                 xmlSecAssert2(ctx->crt->pbCertEncoded != NULL, -1);
                 xmlSecAssert2(ctx->crt->cbCertEncoded > 0, -1);
 
@@ -710,14 +793,14 @@ xmlSecMSCngKeyDataX509Write(xmlSecKeyDataPtr data, xmlSecKeyValueX509Ptr x509Val
                     return(-1);
                 }
             }
-            if ((content & XMLSEC_X509DATA_SKI_NODE) != 0) {
+            if (XMLSEC_X509DATA_HAS_EMPTY_NODE(content, XMLSEC_X509DATA_SKI_NODE)) {
                 ret = xmlSecMSCngX509SKIWrite(ctx->crt, &(x509Value->ski));
                 if (ret < 0) {
                     xmlSecInternalError("xmlSecMSCngX509SKIWrite", xmlSecKeyDataGetName(data));
                     return(-1);
                 }
             }
-            if ((content & XMLSEC_X509DATA_SUBJECTNAME_NODE) != 0) {
+            if (XMLSEC_X509DATA_HAS_EMPTY_NODE(content, XMLSEC_X509DATA_SUBJECTNAME_NODE)) {
                 xmlSecAssert2(x509Value->subject == NULL, -1);
                 xmlSecAssert2(ctx->crt->pCertInfo != NULL, -1);
 
@@ -727,7 +810,7 @@ xmlSecMSCngKeyDataX509Write(xmlSecKeyDataPtr data, xmlSecKeyValueX509Ptr x509Val
                     return(-1);
                 }
             }
-            if ((content & XMLSEC_X509DATA_ISSUERSERIAL_NODE) != 0) {
+            if (XMLSEC_X509DATA_HAS_EMPTY_NODE(content, XMLSEC_X509DATA_ISSUERSERIAL_NODE)) {
                 xmlSecAssert2(x509Value->issuerName == NULL, -1);
                 xmlSecAssert2(x509Value->issuerSerial == NULL, -1);
                 xmlSecAssert2(ctx->crt->pCertInfo != NULL, -1);
@@ -743,8 +826,15 @@ xmlSecMSCngKeyDataX509Write(xmlSecKeyDataPtr data, xmlSecKeyValueX509Ptr x509Val
                    return(-1);
                 }
             }
+            if( (XMLSEC_X509DATA_HAS_EMPTY_NODE(content, XMLSEC_X509DATA_DIGEST_NODE)) && (x509Value->digestAlgorithm != NULL)) {
+                ret = xmlSecMSCngX509DigestWrite(ctx->crt, x509Value->digestAlgorithm, &(x509Value->digest));
+                if (ret < 0) {
+                    xmlSecInternalError("xmlSecMSCngX509DigestWrite", xmlSecKeyDataGetName(data));
+                    return(-1);
+                }
+            }
             /* done */
-            return(0);
+            return(1);
         } else {
             ctx->doneCrts = 1;
         }
@@ -765,7 +855,7 @@ xmlSecMSCngKeyDataX509Write(xmlSecKeyDataPtr data, xmlSecKeyValueX509Ptr x509Val
                 }
             }
             /* done */
-            return(0);
+            return(1);
         } else {
             ctx->doneCrls = 1;
         }
@@ -774,7 +864,7 @@ xmlSecMSCngKeyDataX509Write(xmlSecKeyDataPtr data, xmlSecKeyValueX509Ptr x509Val
     /* no more certs or crls */
     xmlSecAssert2(ctx->doneCrts != 0, -1);
     xmlSecAssert2(ctx->doneCrls != 0, -1);
-    return(1);
+    return(0);
 }
 
 static int
@@ -840,7 +930,7 @@ static xmlSecKeyDataKlass xmlSecMSCngKeyDataX509Klass = {
 
     /* data */
     xmlSecNameX509Data,
-    xmlSecKeyDataUsageKeyInfoNode | xmlSecKeyDataUsageRetrievalMethodNodeXml,
+    xmlSecKeyDataUsageReadFromFile | xmlSecKeyDataUsageKeyInfoNode | xmlSecKeyDataUsageRetrievalMethodNodeXml,
                                                 /* xmlSecKeyDataUsage usage; */
     xmlSecHrefX509Data,                         /* const xmlChar* href; */
     xmlSecNodeX509Data,                         /* const xmlChar* dataNodeName; */
@@ -910,29 +1000,35 @@ xmlSecMSCngKeyDataRawX509CertBinRead(xmlSecKeyDataId id, xmlSecKeyPtr key,
         return(-1);
     }
 
-    data = xmlSecKeyEnsureData(key, xmlSecMSCngKeyDataX509Id);
+    data = xmlSecKeyDataCreate(xmlSecMSCngKeyDataX509Id);
     if(data == NULL) {
-        xmlSecInternalError("xmlSecKeyEnsureData",
-            xmlSecKeyDataKlassGetName(id));
+        xmlSecInternalError("xmlSecKeyDataCreate(xmlSecKeyDataCreate)", xmlSecKeyDataKlassGetName(id));
         CertFreeCertificateContext(cert);
         return(-1);
     }
 
     ret = xmlSecMSCngKeyDataX509AdoptCert(data, cert);
     if(ret < 0) {
-        xmlSecInternalError("xmlSecMSCngKeyDataX509AdoptCert",
-            xmlSecKeyDataKlassGetName(id));
+        xmlSecInternalError("xmlSecMSCngKeyDataX509AdoptCert", xmlSecKeyDataKlassGetName(id));
         CertFreeCertificateContext(cert);
+        xmlSecKeyDataDestroy(data);
         return(-1);
     }
+    cert = NULL; /* owned by data now */
 
-    ret = xmlSecMSCngKeyDataX509VerifyAndExtractKey(data, key, keyInfoCtx);
+    ret = xmlSecMSCnVerifyAndAdoptX509KeyData(key, data, keyInfoCtx);
     if(ret < 0) {
-        xmlSecInternalError("xmlSecMSCngKeyDataX509VerifyAndExtractKey",
-            xmlSecKeyDataKlassGetName(id));
+        xmlSecInternalError("xmlSecMSCnVerifyAndAdoptX509KeyData", xmlSecKeyDataKlassGetName(id));
+        xmlSecKeyDataDestroy(data);
         return(-1);
+    } else if (ret != 1) {
+        /* no errors but key was not found and data was not adopted */
+        xmlSecKeyDataDestroy(data);
+        return(0);
     }
+    data = NULL; /* owned by data now */
 
+    /* success */
     return(0);
 }
 
@@ -942,7 +1038,7 @@ static xmlSecKeyDataKlass xmlSecMSCngKeyDataRawX509CertKlass = {
 
     /* data */
     xmlSecNameRawX509Cert,
-    xmlSecKeyDataUsageRetrievalMethodNodeBin,
+    xmlSecKeyDataUsageReadFromFile | xmlSecKeyDataUsageRetrievalMethodNodeBin,
                                                 /* xmlSecKeyDataUsage usage; */
     xmlSecHrefRawX509Cert,                      /* const xmlChar* href; */
     NULL,                                       /* const xmlChar* dataNodeName; */
