@@ -1,17 +1,16 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
+ * X509 certificates verification support functions for MSCng.
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  * Copyright (C) 2018 Miklos Vajna. All Rights Reserved.
  */
 /**
- * SECTION:x509vfy
- * @Short_description: X509 certificates verification support functions for Microsoft Cryptography API: Next Generation (CNG).
- * @Stability: Private
- *
+ * SECTION:x509
  */
 
 #include "globals.h"
@@ -35,6 +34,7 @@
 #include <xmlsec/mscng/x509.h>
 
 #include "../cast_helpers.h"
+#include "private.h"
 
 typedef struct _xmlSecMSCngX509StoreCtx xmlSecMSCngX509StoreCtx,
                                        *xmlSecMSCngX509StoreCtxPtr;
@@ -48,9 +48,6 @@ struct _xmlSecMSCngX509StoreCtx {
 XMLSEC_KEY_DATA_STORE_DECLARE(MSCngX509Store, xmlSecMSCngX509StoreCtx)
 #define xmlSecMSCngX509StoreSize XMLSEC_KEY_DATA_STORE_SIZE(MSCngX509Store)
 
-static PCCERT_CONTEXT xmlSecMSCngX509FindCertByIssuerNameAndSerial        (HCERTSTORE store,
-                                                                           const xmlChar* issuerName,
-                                                                           const xmlChar* issuerSerial);
 
 static void
 xmlSecMSCngX509StoreFinalize(xmlSecKeyDataStorePtr store) {
@@ -391,42 +388,6 @@ xmlSecMSCngCheckRevocation(HCERTSTORE store, PCCERT_CONTEXT cert) {
     return(0);
 }
 
-/* this function does NOT check for time validity (see xmlSecMSCngVerifyCertTime)
-*  returns <0 if there is an error; 0 if verification failed and >0 if verification succeeded */
-static int
-xmlSecMSCngX509StoreVerifySubject(PCCERT_CONTEXT cert, PCCERT_CONTEXT issuerCert) {
-    DWORD flags;
-    BOOL ret;
-
-    xmlSecAssert2(cert != NULL, -1);
-    xmlSecAssert2(issuerCert != NULL, -1);
-
-    flags = CERT_STORE_REVOCATION_FLAG | CERT_STORE_SIGNATURE_FLAG;
-    ret = CertVerifySubjectCertificateContext(cert, issuerCert, &flags);
-    if (!ret) {
-        xmlSecMSCngLastError("CertVerifySubjectCertificateContext", NULL);
-        return(-1);
-    }
-
-    /* parse returned flags: https://learn.microsoft.com/en-us/previous-versions/windows/embedded/ms883939(v=msdn.10) */
-    if ((flags & CERT_STORE_SIGNATURE_FLAG) != 0) {
-        xmlSecOtherError(XMLSEC_ERRORS_R_CERT_VERIFY_FAILED,
-            NULL,
-            "CertVerifySubjectCertificateContext: CERT_STORE_SIGNATURE_FLAG");
-        return(0);
-    } else if (((flags & CERT_STORE_REVOCATION_FLAG) != 0) && ((flags & CERT_STORE_NO_CRL_FLAG) == 0)) {
-        /* If CERT_STORE_REVOCATION_FLAG is enabled and the issuer does not have a CRL in the store,
-        then CERT_STORE_NO_CRL_FLAG is set in addition to CERT_STORE_REVOCATION_FLAG. */
-        xmlSecOtherError(XMLSEC_ERRORS_R_CERT_VERIFY_FAILED,
-            NULL,
-            "CertVerifySubjectCertificateContext: CERT_STORE_REVOCATION_FLAG");
-        return(0);
-    }
-
-    /* success */
-    return(1);
-}
-
 /**
  * xmlSecMSCngX509StoreContainsCert:
  * @store: the certificate store
@@ -442,39 +403,37 @@ static int
 xmlSecMSCngX509StoreContainsCert(HCERTSTORE store, CERT_NAME_BLOB* name,
         PCCERT_CONTEXT cert)
 {
-    PCCERT_CONTEXT storeCert = NULL;
+    PCCERT_CONTEXT issuerCert = NULL;
+    DWORD flags;
     int ret;
 
     xmlSecAssert2(store != NULL, -1);
     xmlSecAssert2(name != NULL, -1);
     xmlSecAssert2(cert != NULL, -1);
 
-    storeCert = CertFindCertificateInStore(store,
+    issuerCert = CertFindCertificateInStore(store,
         X509_ASN_ENCODING | PKCS_7_ASN_ENCODING,
         0,
         CERT_FIND_SUBJECT_NAME,
         name,
         NULL);
-    if (storeCert == NULL) {
-        return (0);
+    if(issuerCert != NULL) {
+        flags = CERT_STORE_REVOCATION_FLAG | CERT_STORE_SIGNATURE_FLAG;
+        ret = CertVerifySubjectCertificateContext(cert,
+            issuerCert,
+            &flags);
+        if(ret == 0) {
+            xmlSecOtherError(XMLSEC_ERRORS_R_CERT_VERIFY_FAILED,
+                NULL,
+                "CertVerifySubjectCertificateContext");
+            CertFreeCertificateContext(issuerCert);
+            return(-1);
+        }
+        CertFreeCertificateContext(issuerCert);
+        return(1);
     }
 
-    ret = xmlSecMSCngX509StoreVerifySubject(cert, storeCert);
-    if (ret < 0) {
-        xmlSecInternalError("xmlSecMSCngX509StoreVerifySubject", NULL);
-        CertFreeCertificateContext(storeCert);
-        return(-1);
-    } else if (ret == 0) {
-        xmlSecOtherError(XMLSEC_ERRORS_R_CERT_VERIFY_FAILED,
-            NULL,
-            "xmlSecMSCngX509StoreVerifySubject");
-        CertFreeCertificateContext(storeCert);
-        return(-1);
-    }
-
-    /* success */
-    CertFreeCertificateContext(storeCert);
-    return(1);
+    return(0);
 }
 
 static int
@@ -483,14 +442,14 @@ xmlSecMSCngVerifyCertTime(PCCERT_CONTEXT cert, LPFILETIME time) {
     xmlSecAssert2(cert->pCertInfo != NULL, -1);
     xmlSecAssert2(time != NULL, -1);
 
-    if(CompareFileTime(&(cert->pCertInfo->NotBefore), time) == 1) {
+    if(CompareFileTime(&cert->pCertInfo->NotBefore, time) == 1) {
         xmlSecOtherError(XMLSEC_ERRORS_R_CERT_VERIFY_FAILED,
             NULL,
             "CompareFileTime");
         return(-1);
     }
 
-    if(CompareFileTime(&(cert->pCertInfo->NotAfter), time) == -1) {
+    if(CompareFileTime(&cert->pCertInfo->NotAfter, time) == -1) {
         xmlSecOtherError(XMLSEC_ERRORS_R_CERT_VERIFY_FAILED,
             NULL,
             "CompareFileTime");
@@ -517,6 +476,7 @@ xmlSecMSCngX509StoreVerifyCertificateOwn(PCCERT_CONTEXT cert,
         FILETIME* time, HCERTSTORE trustedStore, HCERTSTORE untrustedStore, HCERTSTORE certStore,
         xmlSecKeyDataStorePtr store) {
     PCCERT_CONTEXT issuerCert = NULL;
+    DWORD flags;
     int ret;
 
     xmlSecAssert2(cert != NULL, -1);
@@ -524,7 +484,6 @@ xmlSecMSCngX509StoreVerifyCertificateOwn(PCCERT_CONTEXT cert,
     xmlSecAssert2(certStore != NULL, -1);
     xmlSecAssert2(store != NULL, -1);
 
-    /* check certificate validity and revokation */
     ret = xmlSecMSCngVerifyCertTime(cert, time);
     if(ret < 0) {
         xmlSecInternalError("xmlSecMSCngVerifyCertTime",
@@ -541,33 +500,32 @@ xmlSecMSCngX509StoreVerifyCertificateOwn(PCCERT_CONTEXT cert,
 
     /* does trustedStore contain cert directly? */
     ret = xmlSecMSCngX509StoreContainsCert(trustedStore,
-        &(cert->pCertInfo->Subject), cert);
+        &cert->pCertInfo->Subject, cert);
     if(ret < 0) {
         xmlSecInternalError("xmlSecMSCngX509StoreContainsCert",
             xmlSecKeyDataStoreGetName(store));
         return(-1);
-    } else if(ret == 1) {
-        /* success */
+    }
+    if(ret == 1) {
         return(0);
     }
 
     /* does trustedStore contain the issuer cert? */
     ret = xmlSecMSCngX509StoreContainsCert(trustedStore,
-        &(cert->pCertInfo->Issuer), cert);
+        &cert->pCertInfo->Issuer, cert);
     if(ret < 0) {
         xmlSecInternalError("xmlSecMSCngX509StoreContainsCert",
             xmlSecKeyDataStoreGetName(store));
         return(-1);
-    } else if(ret == 1) {
-        /* success */
+    }
+    if(ret == 1) {
         return(0);
     }
 
     /* is cert self-signed? no recursion in that case */
     if(CertCompareCertificateName(X509_ASN_ENCODING | PKCS_7_ASN_ENCODING,
-            &(cert->pCertInfo->Subject),
-            &(cert->pCertInfo->Issuer))) {
-        /* not verified */
+            &cert->pCertInfo->Subject,
+            &cert->pCertInfo->Issuer)) {
         return(-1);
     }
 
@@ -576,19 +534,15 @@ xmlSecMSCngX509StoreVerifyCertificateOwn(PCCERT_CONTEXT cert,
         X509_ASN_ENCODING | PKCS_7_ASN_ENCODING,
         0,
         CERT_FIND_SUBJECT_NAME,
-        &(cert->pCertInfo->Issuer),
+        &cert->pCertInfo->Issuer,
         NULL);
     if(issuerCert != NULL) {
-        ret = xmlSecMSCngX509StoreVerifySubject(cert, issuerCert);
-        if (ret < 0) {
-            xmlSecInternalError("xmlSecMSCngX509StoreVerifySubject", NULL);
-            CertFreeCertificateContext(issuerCert);
-            return(-1);
-        }
-        else if (ret == 0) {
+        flags = CERT_STORE_REVOCATION_FLAG | CERT_STORE_SIGNATURE_FLAG;
+        ret = CertVerifySubjectCertificateContext(cert, issuerCert, &flags);
+        if(ret == 0) {
             xmlSecOtherError(XMLSEC_ERRORS_R_CERT_VERIFY_FAILED,
-                NULL,
-                "xmlSecMSCngX509StoreVerifySubject");
+                xmlSecKeyDataStoreGetName(store),
+                "CertVerifySubjectCertificateContext");
             CertFreeCertificateContext(issuerCert);
             return(-1);
         }
@@ -609,19 +563,15 @@ xmlSecMSCngX509StoreVerifyCertificateOwn(PCCERT_CONTEXT cert,
         X509_ASN_ENCODING | PKCS_7_ASN_ENCODING,
         0,
         CERT_FIND_SUBJECT_NAME,
-        &(cert->pCertInfo->Issuer),
+        &cert->pCertInfo->Issuer,
         NULL);
     if(issuerCert != NULL) {
-        ret = xmlSecMSCngX509StoreVerifySubject(cert, issuerCert);
-        if (ret < 0) {
-            xmlSecInternalError("xmlSecMSCngX509StoreVerifySubject", NULL);
-            CertFreeCertificateContext(issuerCert);
-            return(-1);
-        }
-        else if (ret == 0) {
+        flags = CERT_STORE_REVOCATION_FLAG | CERT_STORE_SIGNATURE_FLAG;
+        ret = CertVerifySubjectCertificateContext(cert, issuerCert, &flags);
+        if(ret == 0) {
             xmlSecOtherError(XMLSEC_ERRORS_R_CERT_VERIFY_FAILED,
-                NULL,
-                "xmlSecMSCngX509StoreVerifySubject");
+                xmlSecKeyDataStoreGetName(store),
+                "CertVerifySubjectCertificateContext");
             CertFreeCertificateContext(issuerCert);
             return(-1);
         }
@@ -747,7 +697,7 @@ xmlSecMSCngUnixTimeToFileTime(time_t in, LPFILETIME out) {
  * @store: the pointer to X509 certificate context store klass.
  * @cert: the certificate to verify.
  * @certStore: the untrusted certificates stack.
- * @keyInfoCtx: the pointer to <dsig:KeyInfo/> element processing context.
+ * @keyInfoCtx: the pointer to &lt;dsig:KeyInfo/&gt; element processing context.
  *
  * Verifies @cert.
  *
@@ -799,7 +749,7 @@ xmlSecMSCngX509StoreVerifyCertificate(xmlSecKeyDataStorePtr store,
  * xmlSecMSCngX509StoreVerify:
  * @store: the pointer to X509 certificate context store klass.
  * @certs: the untrusted certificates stack.
- * @keyInfoCtx: the pointer to <dsig:KeyInfo/> element processing context.
+ * @keyInfoCtx: the pointer to &lt;dsig:KeyInfo/&gt; element processing context.
  *
  * Verifies @certs list.
  *
@@ -918,51 +868,20 @@ xmlSecMSCngCertStrToName(DWORD dwCertEncodingType, LPTSTR pszX500, DWORD dwStrTy
 }
 
 static PCCERT_CONTEXT
-xmlSecMSCngX509FindCertByIssuerNameAndSerial(HCERTSTORE store, const xmlChar* issuerName, const xmlChar* issuerSerial) {
+xmlSecMSCngX509FindCertByIssuerNameAndSerial(HCERTSTORE store, LPTSTR wcIssuerName, xmlSecBnPtr issuerSerialBn, DWORD dwCertEncodingType) {
     PCCERT_CONTEXT res = NULL;
-    xmlSecBn issuerSerialBn;
-    int issuerSerialBnInitialized = 0;
-    LPTSTR wcIssuerName = NULL;
-    DWORD dwCertEncodingType = X509_ASN_ENCODING | PKCS_7_ASN_ENCODING;
     CERT_INFO certInfo;
     BYTE* bdata = NULL;
     xmlSecSize issuerSerialSize;
     DWORD len;
-    int ret;
 
     xmlSecAssert2(store != 0, NULL);
-    xmlSecAssert2(issuerName != NULL, NULL);
-    xmlSecAssert2(issuerSerial != NULL, NULL);
+    xmlSecAssert2(wcIssuerName != NULL, NULL);
+    xmlSecAssert2(issuerSerialBn != NULL, NULL);
 
-    ret = xmlSecBnInitialize(&issuerSerialBn, 0);
-    if (ret < 0) {
-        xmlSecInternalError("xmlSecBnInitialize", NULL);
-        goto done;
-    }
-    issuerSerialBnInitialized = 1;
-
-    ret = xmlSecBnFromDecString(&issuerSerialBn, issuerSerial);
-    if (ret < 0) {
-        xmlSecInternalError("xmlSecBnFromDecString", NULL);
-        goto done;
-    }
-
-    /* MS Windows wants this in the opposite order */
-    ret = xmlSecBnReverse(&issuerSerialBn);
-    if (ret < 0) {
-        xmlSecInternalError("xmlSecBnReverse", NULL);
-        goto done;
-    }
-
-    certInfo.SerialNumber.pbData = xmlSecBnGetData(&issuerSerialBn);
-    issuerSerialSize  = xmlSecBnGetSize(&issuerSerialBn);
-    XMLSEC_SAFE_CAST_SIZE_TO_ULONG(issuerSerialSize, certInfo.SerialNumber.cbData, goto done, NULL);
-
-    wcIssuerName = xmlSecMSCngX509GetCertName(issuerName);
-    if (wcIssuerName == NULL) {
-        xmlSecInternalError("xmlSecMSCngX509GetCertName", NULL);
-        goto done;
-    }
+    certInfo.SerialNumber.pbData = xmlSecBnGetData(issuerSerialBn);
+    issuerSerialSize  = xmlSecBnGetSize(issuerSerialBn);
+    XMLSEC_SAFE_CAST_SIZE_TO_ULONG(issuerSerialSize, certInfo.SerialNumber.cbData, return(NULL), NULL);
 
     /* CASE 1: UTF8, DN */
     if (NULL == res) {
@@ -1048,71 +967,142 @@ xmlSecMSCngX509FindCertByIssuerNameAndSerial(HCERTSTORE store, const xmlChar* is
         }
     }
 
-done:
     if (bdata != NULL) {
         xmlFree(bdata);
     }
-    if (wcIssuerName != NULL) {
-        xmlFree(wcIssuerName);
-    }
-    if (issuerSerialBnInitialized) {
-        xmlSecBnFinalize(&issuerSerialBn);
-    }
     return(res);
 }
 
 static PCCERT_CONTEXT
-xmlSecMSCngX509FindCertBySki(HCERTSTORE store, xmlSecByte* ski, xmlSecSize skiSize) {
+xmlSecMSCngX509FindCertBySki(HCERTSTORE store, const xmlSecByte* ski, DWORD skiLen, DWORD dwCertEncodingType) {
     CRYPT_HASH_BLOB blob;
 
     xmlSecAssert2(store != 0, NULL);
     xmlSecAssert2(ski != NULL, NULL);
-    xmlSecAssert2(skiSize > 0, NULL);
+    xmlSecAssert2(skiLen > 0, NULL);
 
-    blob.pbData = ski;
-    XMLSEC_SAFE_CAST_SIZE_TO_ULONG(skiSize, blob.cbData, return(NULL), NULL);
+    blob.pbData = (PBYTE)ski; /* remove const */
+    blob.cbData = skiLen;
 
     return(CertFindCertificateInStore(store,
-        PKCS_7_ASN_ENCODING | X509_ASN_ENCODING,
+        dwCertEncodingType,
         0,
         CERT_FIND_KEY_IDENTIFIER,
         &blob,
         NULL));
 }
 
+/* ONLY SHA1 DIGEST IS CURRENTLY SUPPORTED */
 static PCCERT_CONTEXT
-xmlSecMSCngX509FindCert(HCERTSTORE store, xmlChar* subjectName,
-                        xmlChar* issuerName, xmlChar* issuerSerial,
-                        xmlSecByte* ski, xmlSecSize skiSize) {
+xmlSecMSCngX509FindCertByDigest(HCERTSTORE store, const xmlSecByte* digest, DWORD digestLen, DWORD dwCertEncodingType) {
+    CRYPT_HASH_BLOB blob;
+
+    xmlSecAssert2(store != 0, NULL);
+    xmlSecAssert2(digest != NULL, NULL);
+    xmlSecAssert2(digestLen > 0, NULL);
+
+    blob.pbData = (PBYTE)digest; /* remove const */
+    blob.cbData = digestLen;
+
+    return(CertFindCertificateInStore(store,
+        dwCertEncodingType,
+        0,
+        CERT_FIND_SHA1_HASH,
+        &blob,
+        NULL));
+}
+
+PCCERT_CONTEXT
+xmlSecMSCngX509FindCert(HCERTSTORE store, xmlSecMSCngX509FindCertCtxPtr findCertCtx) {
+    DWORD dwCertEncodingType = X509_ASN_ENCODING | PKCS_7_ASN_ENCODING;
     PCCERT_CONTEXT cert = NULL;
 
     xmlSecAssert2(store != 0, NULL);
+    xmlSecAssert2(findCertCtx != 0, NULL);
 
-    if(subjectName != NULL) {
-        LPTSTR wcSubjectName;
-
-        wcSubjectName = xmlSecMSCngX509GetCertName(subjectName);
-        if(wcSubjectName == NULL) {
-            xmlSecInternalError("xmlSecMSCngX509GetCertName", NULL);
-            return(NULL);
-        }
-
-        cert = xmlSecMSCngX509FindCertBySubject(store, wcSubjectName,
-            PKCS_7_ASN_ENCODING | X509_ASN_ENCODING);
-        xmlFree(wcSubjectName);
+    if((cert == NULL) && (findCertCtx->wcSubjectName != NULL)) {
+        cert = xmlSecMSCngX509FindCertBySubject(store, findCertCtx->wcSubjectName, dwCertEncodingType);
     }
 
-    if(issuerName != NULL && issuerSerial != NULL) {
-        cert = xmlSecMSCngX509FindCertByIssuerNameAndSerial(store, issuerName, issuerSerial);
+    if((cert == NULL) && (findCertCtx->wcIssuerName != NULL) && (findCertCtx->issuerSerialBn != NULL)) {
+        cert = xmlSecMSCngX509FindCertByIssuerNameAndSerial(store, findCertCtx->wcIssuerName, findCertCtx->issuerSerialBn, dwCertEncodingType);
     }
 
-    if((ski != NULL) && (skiSize > 0)) {
-        cert = xmlSecMSCngX509FindCertBySki(store, ski, skiSize);
+    if((cert == NULL) &&  (findCertCtx->ski != NULL) && (findCertCtx->skiLen > 0)) {
+        cert = xmlSecMSCngX509FindCertBySki(store, findCertCtx->ski, findCertCtx->skiLen, dwCertEncodingType);
+    }
+    if ((cert == NULL) && (findCertCtx->digestValue != NULL) && (findCertCtx->digestLen > 0)) {
+        cert = xmlSecMSCngX509FindCertByDigest(store, findCertCtx->digestValue, findCertCtx->digestLen, dwCertEncodingType);
     }
 
     return(cert);
 }
 
+/* caller must free returned string with xmlFree() */
+LPCWSTR
+xmlSecMSCngX509GetFriendlyNameUnicode(PCCERT_CONTEXT cert) {
+    DWORD dwPropSize;
+    PBYTE pbFriendlyName;
+    BOOL ret;
+
+    xmlSecAssert2(cert != 0, NULL);
+
+    /* CERT_FRIENDLY_NAME_PROP_ID: Returns a null-terminated Unicode character
+     * string that contains the display name for the certificate. */
+    ret = CertGetCertificateContextProperty(cert,
+        CERT_FRIENDLY_NAME_PROP_ID,
+        NULL, &dwPropSize);
+    if (ret != TRUE) {
+        /* name might not exists */
+        return(NULL);
+    }
+
+    pbFriendlyName = xmlMalloc(dwPropSize);
+    if (pbFriendlyName == NULL) {
+        xmlSecMallocError(dwPropSize, NULL);
+        return(NULL);
+    }
+
+    ret = CertGetCertificateContextProperty(cert,
+        CERT_FRIENDLY_NAME_PROP_ID,
+        pbFriendlyName,
+        &dwPropSize);
+    if ((ret != TRUE) || (dwPropSize <= 0)) {
+        xmlSecMSCngLastError("CertGetCertificateContextProperty", NULL);
+        return(NULL);
+    }
+
+    /* success: always unicode string! */
+    return((LPCWSTR)pbFriendlyName);
+}
+
+/* caller must free returned string with xmlFree() */
+xmlChar*
+xmlSecMSCngX509GetFriendlyNameUtf8(PCCERT_CONTEXT cert) {
+    LPCWSTR str;
+    xmlChar* res;
+
+    xmlSecAssert2(cert != 0, NULL);
+
+    str = xmlSecMSCngX509GetFriendlyNameUnicode(cert);
+    if (str == NULL) {
+        /* name might not exists */
+        return(NULL);
+    }
+
+    /* convert name to utf8 */
+    res = xmlSecWin32ConvertUnicodeToUtf8(str);
+    if (res == NULL) {
+        xmlSecInternalError("xmlSecWin32ConvertUnicodeToUtf8", NULL);
+        xmlFree((void*)str);
+        return(NULL);
+    }
+
+    /* success */
+    xmlFree((void*)str);
+    return(res);
+}
+
 /**
  * xmlSecMSCngX509StoreFindCert:
  * @store:          the pointer to X509 key data store klass.
@@ -1120,7 +1110,7 @@ xmlSecMSCngX509FindCert(HCERTSTORE store, xmlChar* subjectName,
  * @issuerName:     the desired certificate issuer name.
  * @issuerSerial:   the desired certificate issuer serial number.
  * @ski:            the desired certificate SKI.
- * @keyInfoCtx:     the pointer to <dsig:KeyInfo/> element processing context.
+ * @keyInfoCtx:     the pointer to &lt;dsig:KeyInfo/&gt; element processing context.
  *
  * Searches @store for a certificate that matches given criteria.
  *
@@ -1160,7 +1150,7 @@ xmlSecMSCngX509StoreFindCert(xmlSecKeyDataStorePtr store, xmlChar *subjectName,
  * @issuerSerial:   the desired certificate issuer serial number.
  * @ski:            the desired certificate SKI.
  * @skiSize:        the desired certificate SKI size.
- * @keyInfoCtx:     the pointer to <dsig:KeyInfo/> element processing context.
+ * @keyInfoCtx:     the pointer to &lt;dsig:KeyInfo/&gt; element processing context.
  *
  * Searches @store for a certificate that matches given criteria.
  *
@@ -1172,8 +1162,10 @@ xmlSecMSCngX509StoreFindCert_ex(xmlSecKeyDataStorePtr store, xmlChar* subjectNam
                                 xmlChar* issuerName, xmlChar* issuerSerial,
                                 xmlSecByte* ski, xmlSecSize skiSize,
                                 xmlSecKeyInfoCtx* keyInfoCtx ATTRIBUTE_UNUSED) {
+    xmlSecMSCngX509FindCertCtx findCertCtx;
     xmlSecMSCngX509StoreCtxPtr ctx;
     PCCERT_CONTEXT cert = NULL;
+    int ret;
 
     xmlSecAssert2(xmlSecKeyDataStoreCheckId(store, xmlSecMSCngX509StoreId), NULL);
     UNREFERENCED_PARAMETER(keyInfoCtx);
@@ -1181,21 +1173,67 @@ xmlSecMSCngX509StoreFindCert_ex(xmlSecKeyDataStorePtr store, xmlChar* subjectNam
     ctx = xmlSecMSCngX509StoreGetCtx(store);
     xmlSecAssert2(ctx != NULL, NULL);
 
+    ret = xmlSecMSCngX509FindCertCtxInitialize(&findCertCtx,
+        subjectName,
+        issuerName, issuerSerial,
+        ski, skiSize);
+    if (ret < 0) {
+        xmlSecInternalError("xmlSecMSCngX509FindCertCtxInitialize", NULL);
+        xmlSecMSCngX509FindCertCtxFinalize(&findCertCtx);
+        return(NULL);
+    }
+
     /* search untrusted certs store */
-    if (ctx->untrusted != NULL) {
-        cert = xmlSecMSCngX509FindCert(ctx->untrusted, subjectName,
-            issuerName, issuerSerial, ski, skiSize);
+    if ((cert == NULL) && (ctx->untrusted != NULL)) {
+        cert = xmlSecMSCngX509FindCert(ctx->untrusted, &findCertCtx);
     }
 
     /* search trusted certs store */
-    if (cert == NULL && ctx->trusted != NULL) {
-        cert = xmlSecMSCngX509FindCert(ctx->trusted, subjectName,
-            issuerName, issuerSerial, ski, skiSize);
+    if ((cert == NULL) && (ctx->trusted != NULL)) {
+        cert = xmlSecMSCngX509FindCert(ctx->trusted, &findCertCtx);
     }
 
+    /* done */
+    xmlSecMSCngX509FindCertCtxFinalize(&findCertCtx);
     return(cert);
 }
 
+PCCERT_CONTEXT
+xmlSecMSCngX509StoreFindCertByValue(xmlSecKeyDataStorePtr store, xmlSecKeyX509DataValuePtr x509Value) {
+    xmlSecMSCngX509FindCertCtx findCertCtx;
+    xmlSecMSCngX509StoreCtxPtr ctx;
+    PCCERT_CONTEXT cert = NULL;
+    int ret;
+
+    xmlSecAssert2(xmlSecKeyDataStoreCheckId(store, xmlSecMSCngX509StoreId), NULL);
+    xmlSecAssert2(x509Value != NULL, NULL);
+
+    ctx = xmlSecMSCngX509StoreGetCtx(store);
+    xmlSecAssert2(ctx != NULL, NULL);
+
+    ret = xmlSecMSCngX509FindCertCtxInitializeFromValue(&findCertCtx, x509Value);
+    if (ret < 0) {
+        xmlSecInternalError("xmlSecMSCngX509FindCertCtxInitializeFromValue", NULL);
+        xmlSecMSCngX509FindCertCtxFinalize(&findCertCtx);
+        return(NULL);
+    }
+
+    /* search untrusted certs store */
+    if ((cert == NULL) && (ctx->untrusted != NULL)) {
+        cert = xmlSecMSCngX509FindCert(ctx->untrusted, &findCertCtx);
+    }
+
+    /* search trusted certs store */
+    if ((cert == NULL) && (ctx->trusted != NULL)) {
+        cert = xmlSecMSCngX509FindCert(ctx->trusted, &findCertCtx);
+    }
+
+    /* done */
+    xmlSecMSCngX509FindCertCtxFinalize(&findCertCtx);
+    return(cert);
+
+}
+
 /**
  * xmlSecMSCngX509FindCertBySubject:
  * @store:              the pointer to certs store
@@ -1300,4 +1338,121 @@ xmlSecMSCngX509FindCertBySubject(HCERTSTORE store, LPTSTR wcSubject,
     return(res);
 }
 
+
+/******************************************************************************
+ *
+ * xmlSecMSCngX509FindCert functions
+ *
+ ******************************************************************************/
+int
+xmlSecMSCngX509FindCertCtxInitialize(xmlSecMSCngX509FindCertCtxPtr ctx,
+    const xmlChar* subjectName,
+    const xmlChar* issuerName, const xmlChar* issuerSerial,
+    const xmlSecByte* ski, xmlSecSize skiSize
+) {
+    int ret;
+    xmlSecAssert2(ctx != NULL, -1);
+
+    memset(ctx, 0, sizeof(*ctx));
+
+    /* simplest one first */
+    if ((ski != NULL) && (skiSize > 0)) {
+        ctx->ski = ski;
+        XMLSEC_SAFE_CAST_SIZE_TO_UINT(skiSize, ctx->skiLen, return(-1), NULL);
+    }
+
+    if (subjectName != NULL) {
+        ctx->wcSubjectName = xmlSecMSCngX509GetCertName(subjectName);
+        if (ctx->wcSubjectName == NULL) {
+            xmlSecInternalError("xmlSecMSCngX509GetCertName(subject)", NULL);
+            xmlSecMSCngX509FindCertCtxFinalize(ctx);
+            return(-1);
+        }
+    }
+
+    if ((issuerName != NULL) && (issuerSerial != NULL)) {
+        ctx->wcIssuerName = xmlSecMSCngX509GetCertName(issuerName);
+        if (ctx->wcIssuerName == NULL) {
+            xmlSecInternalError("xmlSecMSCngX509GetCertName(issuer)", NULL);
+            xmlSecMSCngX509FindCertCtxFinalize(ctx);
+            return(-1);
+        }
+
+        ctx->issuerSerialBn = xmlSecBnCreate(0);
+        if (ctx->issuerSerialBn == NULL) {
+            xmlSecInternalError("xmlSecBnCreate(issuerSerial)", NULL);
+            xmlSecMSCngX509FindCertCtxFinalize(ctx);
+            return(-1);
+        }
+        ret = xmlSecBnFromDecString(ctx->issuerSerialBn, issuerSerial);
+        if (ret < 0) {
+            xmlSecInternalError("xmlSecBnFromDecString(issuerSerial)", NULL);
+            xmlSecMSCngX509FindCertCtxFinalize(ctx);
+            return(-1);
+        }
+        /* MS Windows wants this in the opposite order */
+        ret = xmlSecBnReverse(ctx->issuerSerialBn);
+        if (ret < 0) {
+            xmlSecInternalError("xmlSecBnReverse", NULL);
+            xmlSecMSCngX509FindCertCtxFinalize(ctx);
+            return(-1);
+        }
+    }
+
+    /* done! */
+    return(0);
+}
+
+int
+xmlSecMSCngX509FindCertCtxInitializeFromValue(xmlSecMSCngX509FindCertCtxPtr ctx, xmlSecKeyX509DataValuePtr x509Value) {
+    int ret;
+
+    xmlSecAssert2(ctx != NULL, -1);
+    xmlSecAssert2(x509Value != NULL, -1);
+
+    ret = xmlSecMSCngX509FindCertCtxInitialize(ctx,
+        x509Value->subject,
+        x509Value->issuerName, x509Value->issuerSerial,
+        xmlSecBufferGetData(&(x509Value->ski)), xmlSecBufferGetSize(&(x509Value->ski))
+    );
+    if (ret < 0) {
+        xmlSecInternalError("xmlSecMSCngX509FindCertCtxInitialize", NULL);
+        xmlSecMSCngX509FindCertCtxFinalize(ctx);
+        return(-1);
+    }
+
+    if ((!xmlSecBufferIsEmpty(&(x509Value->digest))) && (x509Value->digestAlgorithm != NULL)) {
+        xmlSecSize digestSize;
+
+        /* only SHA1 algorithm is currently supported */
+        if (xmlStrcmp(x509Value->digestAlgorithm, xmlSecHrefSha1) != 0) {
+            xmlSecOtherError2(XMLSEC_ERRORS_R_INVALID_ALGORITHM, NULL,
+                "href=%s", xmlSecErrorsSafeString(x509Value->digestAlgorithm));
+            xmlSecMSCngX509FindCertCtxFinalize(ctx);
+            return(-1);
+        }
+        ctx->digestValue = xmlSecBufferGetData(&(x509Value->digest));
+        digestSize = xmlSecBufferGetSize(&(x509Value->digest));
+        XMLSEC_SAFE_CAST_SIZE_TO_UINT(digestSize, ctx->digestLen, return(-1), NULL);
+    }
+
+    /* done */
+    return(0);
+}
+
+void xmlSecMSCngX509FindCertCtxFinalize(xmlSecMSCngX509FindCertCtxPtr ctx) {
+    xmlSecAssert(ctx != NULL);
+
+    if (ctx->wcSubjectName != NULL) {
+        xmlFree(ctx->wcSubjectName);
+    }
+    if (ctx->wcIssuerName != NULL) {
+        xmlFree(ctx->wcIssuerName);
+    }
+    if (ctx->issuerSerialBn != NULL) {
+        xmlSecBnDestroy(ctx->issuerSerialBn);
+    }
+    memset(ctx, 0, sizeof(*ctx));
+}
+
 #endif /* XMLSEC_NO_X509 */
