@@ -1,6 +1,7 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
+ * Digests transforms implementation for Microsoft Crypto API.
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
@@ -9,10 +10,7 @@
  * Copyright (c) 2005-2006 Cryptocom LTD (http://www.cryptocom.ru).
  */
 /**
- * SECTION:digests
- * @Short_description: Digests transforms implementation for Microsoft Crypto API.
- * @Stability: Private
- *
+ * SECTION:crypto
  */
 
 #include "globals.h"
@@ -71,6 +69,7 @@ static int      xmlSecMSCryptoDigestCheckId     (xmlSecTransformPtr transform);
  *
  * MUST END with { NULL, 0 } !!!
  */
+#ifndef XMLSEC_NO_SHA1
 static xmlSecMSCryptoProviderInfo xmlSecMSCryptoProviderInfo_Sha1[] = {
     { XMLSEC_CRYPTO_MS_ENH_RSA_AES_PROV,                PROV_RSA_AES},
     { XMLSEC_CRYPTO_MS_ENH_RSA_AES_PROV_PROTOTYPE,      PROV_RSA_AES },
@@ -79,6 +78,7 @@ static xmlSecMSCryptoProviderInfo xmlSecMSCryptoProviderInfo_Sha1[] = {
     { MS_DEF_PROV,                                      PROV_RSA_FULL },
     { NULL, 0 }
 };
+#endif /* XMLSEC_NO_SHA1*/
 
 static xmlSecMSCryptoProviderInfo xmlSecMSCryptoProviderInfo_Sha2[] = {
     { XMLSEC_CRYPTO_MS_ENH_RSA_AES_PROV,                PROV_RSA_AES},
@@ -93,7 +93,7 @@ static xmlSecMSCryptoProviderInfo xmlSecMSCryptoProviderInfo_Md5[] = {
     { MS_DEF_PROV,                                      PROV_RSA_FULL },
     { NULL, 0 }
 };
-#endif /* XMLSEC_NO_MD5 */
+#endif /* XMLSEC_NO_MD5*/
 
 #ifndef XMLSEC_NO_GOST
 static xmlSecMSCryptoProviderInfo xmlSecMSCryptoProviderInfo_Gost[] = {
@@ -746,4 +746,3 @@ xmlSecMSCryptoTransformGostR3411_2012_512GetKlass(void) {
     return(&xmlSecMSCryptoGostR3411_2012_512Klass);
 }
 #endif /* XMLSEC_NO_GOST*/
-
