@@ -1,27 +1,15 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
+ * HMAC transforms implementation for Microsoft Crypto API.
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 /**
- * SECTION:hmac
- * @Short_description: HMAC transforms implementation for Microsoft Crypto API.
- * @Stability: Private
- *
- * [HMAC Algorithm support](http://www.w3.org/TR/xmldsig-core/#sec-HMAC):
- * The HMAC algorithm (RFC2104 [HMAC]) takes the truncation length in bits
- * as a parameter; if the parameter is not specified then all the bits of the
- * hash are output. An example of an HMAC SignatureMethod element:
- *
- * |[<!-- language="XML" -->
- * <SignatureMethod Algorithm="http://www.w3.org/2000/09/xmldsig#hmac-sha1">
- *   <HMACOutputLength>128</HMACOutputLength>
- * </SignatureMethod>
- * |]
+ * SECTION:crypto
  */
 
 
@@ -55,41 +43,21 @@
  *
  *****************************************************************************/
 
-/**
- * xmlSecMSCryptoHmacGetMinOutputLength:
- *
- * DEPRECATED (use @xmlSecTransformHmacGetMinOutputBitsSize instead).
- * Gets the value of min HMAC length.
- *
- * Returns: the min HMAC output length
- */
-int xmlSecMSCryptoHmacGetMinOutputLength(void)
-{
-    xmlSecSize val = xmlSecTransformHmacGetMinOutputBitsSize();
-    int res;
-
-    XMLSEC_SAFE_CAST_SIZE_TO_INT(val, res, return(-1), NULL);
-    return res;
-}
-
-/**
- * xmlSecMSCryptoHmacSetMinOutputLength:
- * @min_length: the new min length
- *
- * DEPRECATED (use @xmlSecTransformHmacSetMinOutputBitsSize instead).
- * Sets the min HMAC output length
- */
-void xmlSecMSCryptoHmacSetMinOutputLength(int min_length)
-{
-    xmlSecSize val;
-    XMLSEC_SAFE_CAST_INT_TO_SIZE(min_length, val, return, NULL);
-    xmlSecTransformHmacSetMinOutputBitsSize(val);
-}
-
 /******************************************************************************
  *
  * Internal MSCrypto HMAC CTX
  *
+ * [HMAC Algorithm support](http://www.w3.org/TR/xmldsig-core/#sec-HMAC):
+ * The HMAC algorithm (RFC2104 [HMAC]) takes the truncation length in bits
+ * as a parameter; if the parameter is not specified then all the bits of the
+ * hash are output. An example of an HMAC SignatureMethod element:
+ *
+ * |[<!-- language="XML" -->
+ * <SignatureMethod Algorithm="http://www.w3.org/2000/09/xmldsig#hmac-sha1">
+ *   <HMACOutputLength>128</HMACOutputLength>
+ * </SignatureMethod>
+ * |]
+
  *****************************************************************************/
 typedef struct _xmlSecMSCryptoHmacCtx            xmlSecMSCryptoHmacCtx, *xmlSecMSCryptoHmacCtxPtr;
 struct _xmlSecMSCryptoHmacCtx {
@@ -100,7 +68,8 @@ struct _xmlSecMSCryptoHmacCtx {
     const xmlSecMSCryptoProviderInfo  * providers;
     HCRYPTHASH      mscHash;
     unsigned char   dgst[XMLSEC_TRASNFORM_HMAC_MAX_OUTPUT_SIZE];
-    xmlSecSize      dgstSize;   /* dgst size in bytes */
+    xmlSecSize      dgstSize;
+    xmlSecSize      dgstSizeInBits;   /* dgst size in bytes */
     int             ctxInitialized;
 };
 
@@ -295,7 +264,7 @@ xmlSecMSCryptoHmacNodeRead(xmlSecTransformPtr transform, xmlNodePtr node,
     ctx = xmlSecMSCryptoHmacGetCtx(transform);
     xmlSecAssert2(ctx != NULL, -1);
 
-    ret = xmlSecTransformHmacReadOutputBitsSize(node, ctx->dgstSize, &ctx->dgstSize);
+    ret = xmlSecTransformHmacReadOutputBitsSize(node, ctx->dgstSizeInBits, &ctx->dgstSizeInBits);
     if (ret < 0) {
         xmlSecInternalError("xmlSecTransformHmacReadOutputBitsSize()",
             xmlSecTransformGetName(transform));
@@ -412,11 +381,8 @@ static int
 xmlSecMSCryptoHmacVerify(xmlSecTransformPtr transform,
                         const xmlSecByte* data, xmlSecSize dataSize,
                         xmlSecTransformCtxPtr transformCtx ATTRIBUTE_UNUSED) {
-    static xmlSecByte last_byte_masks[] =
-                { 0xFF, 0x80, 0xC0, 0xE0, 0xF0, 0xF8, 0xFC, 0xFE };
-
     xmlSecMSCryptoHmacCtxPtr ctx;
-    xmlSecByte mask;
+    int ret;
 
     xmlSecAssert2(xmlSecTransformIsValid(transform), -1);
     xmlSecAssert2(xmlSecTransformCheckSize(transform, xmlSecMSCryptoHmacSize), -1);
@@ -427,38 +393,20 @@ xmlSecMSCryptoHmacVerify(xmlSecTransformPtr transform,
 
     ctx = xmlSecMSCryptoHmacGetCtx(transform);
     xmlSecAssert2(ctx != NULL, -1);
-    xmlSecAssert2(ctx->dgstSize > 0, -1);
+    xmlSecAssert2(ctx->dgstSizeInBits > 0, -1);
 
-    /* compare the digest size in bytes */
-    if(dataSize != ((ctx->dgstSize + 7) / 8)){
-        xmlSecInvalidSizeError("HMAC digest",
-                               dataSize, ((ctx->dgstSize + 7) / 8),
-                               xmlSecTransformGetName(transform));
-        transform->status = xmlSecTransformStatusFail;
-        return(0);
+    /* Returns 1 for match, 0 for no match, <0 for errors. */
+    ret = xmlSecTransformHmacVerify(data, dataSize, ctx->dgst, ctx->dgstSizeInBits, ctx->dgstSize);
+    if (ret < 0) {
+        xmlSecInternalError("xmlSecTransformHmacVerify", xmlSecTransformGetName(transform));
+        return(-1);
     }
-
-    /* we check the last byte separately */
-    xmlSecAssert2(dataSize > 0, -1);
-    mask = last_byte_masks[ctx->dgstSize % 8];
-    if((ctx->dgst[dataSize - 1] & mask) != (data[dataSize - 1]  & mask)) {
-        xmlSecOtherError(XMLSEC_ERRORS_R_DATA_NOT_MATCH,
-                         xmlSecTransformGetName(transform),
-                         "data and digest do not match (last byte)");
-        transform->status = xmlSecTransformStatusFail;
-        return(0);
+    if (ret == 1) {
+        transform->status = xmlSecTransformStatusOk;
     }
-
-    /* now check the rest of the digest */
-    if((dataSize > 1) && (memcmp(ctx->dgst, data, dataSize - 1) != 0)) {
-        xmlSecOtherError(XMLSEC_ERRORS_R_DATA_NOT_MATCH,
-                         xmlSecTransformGetName(transform),
-                         "data and digest do not match");
+    else {
         transform->status = xmlSecTransformStatusFail;
-        return(0);
     }
-
-    transform->status = xmlSecTransformStatusOk;
     return(0);
 }
 
@@ -514,7 +462,6 @@ xmlSecMSCryptoHmacExecute(xmlSecTransformPtr transform, int last, xmlSecTransfor
 
         if(last) {
             DWORD retLen = XMLSEC_TRASNFORM_HMAC_MAX_OUTPUT_SIZE;
-            xmlSecSize hashSize;
 
             ret = CryptGetHashParam(ctx->mscHash,
                 HP_HASHVAL,
@@ -528,26 +475,16 @@ xmlSecMSCryptoHmacExecute(xmlSecTransformPtr transform, int last, xmlSecTransfor
                 return(-1);
             }
             xmlSecAssert2(retLen > 0, -1);
-            XMLSEC_SAFE_CAST_ULONG_TO_SIZE(retLen, hashSize, return(-1), xmlSecTransformGetName(transform));
+            XMLSEC_SAFE_CAST_ULONG_TO_SIZE(retLen, ctx->dgstSize, return(-1), xmlSecTransformGetName(transform));
 
             /* check/set the result digest size */
-            if(ctx->dgstSize == 0) {
-                ctx->dgstSize = hashSize * 8; /* no dgst size specified, use all we have */
-            } else if(ctx->dgstSize <= 8 * hashSize) {
-                hashSize = ((ctx->dgstSize + 7) / 8); /* we need to truncate result digest */
-            } else {
-                xmlSecInvalidSizeLessThanError("HMAC digest (bits)",
-                    8 * hashSize, ctx->dgstSize,
-                    xmlSecTransformGetName(transform));
-                return(-1);
+            if(ctx->dgstSizeInBits == 0) {
+                ctx->dgstSizeInBits = ctx->dgstSize * 8; /* no dgst size specified, use all we have */
             }
-
-            /* copy result to output */
-            if(transform->operation == xmlSecTransformOperationSign) {
-                ret = xmlSecBufferAppend(out, ctx->dgst, hashSize);
-                if(ret < 0) {
-                    xmlSecInternalError2("xmlSecBufferAppend", xmlSecTransformGetName(transform),
-                        "size=" XMLSEC_SIZE_FMT, hashSize);
+            if (transform->operation == xmlSecTransformOperationSign) {
+                ret = xmlSecTransformHmacWriteOutput(ctx->dgst, ctx->dgstSizeInBits, ctx->dgstSize, out);
+                if (ret < 0) {
+                    xmlSecInternalError("xmlSecTransformHmacWriteOutput", xmlSecTransformGetName(transform));
                     return(-1);
                 }
             }
@@ -895,4 +832,3 @@ xmlSecMSCryptoTransformHmacSha512GetKlass(void) {
 
 
 #endif /* XMLSEC_NO_HMAC */
-
