@@ -1,17 +1,15 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
+ * DES Key Transport transforms implementation for Microsoft Crypto API.
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 /**
- * SECTION:kw_des
- * @Short_description: DES Key Transport transforms implementation for Microsoft Crypto API.
- * @Stability: Private
- *
+ * SECTION:crypto
  */
 
 #ifndef XMLSEC_NO_DES
@@ -587,5 +585,9 @@ done:
 }
 
 
-#endif /* XMLSEC_NO_DES */
+#else /* XMLSEC_NO_DES */
 
+/* ISO C forbids an empty translation unit */
+typedef int make_iso_compilers_happy;
+
+#endif /* XMLSEC_NO_DES */
