@@ -1,18 +1,20 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
+ * X509 certificates implementation for Microsoft Crypto API.
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
  * Copyright (C) 2003 Cordys R&D BV, All rights reserved.
- * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 /**
  * SECTION:x509
  * @Short_description: X509 certificates implementation for Microsoft Crypto API.
  * @Stability: Stable
  *
+ * X509 certificates implementation for Microsoft Crypto API.
  */
 
 #include "globals.h"
@@ -37,6 +39,7 @@
 #include <xmlsec/x509.h>
 #include <xmlsec/bn.h>
 #include <xmlsec/errors.h>
+#include <xmlsec/private.h>
 #include <xmlsec/xmltree.h>
 
 #include <xmlsec/mscrypto/crypto.h>
@@ -90,7 +93,7 @@ struct _xmlSecMSCryptoX509DataCtx {
 
 /**************************************************************************
  *
- * <dsig:X509Data> processing (http://www.w3.org/TR/xmldsig-core/#sec-X509Data)
+ * &lt;dsig:X509Data/&gt; processing (http://www.w3.org/TR/xmldsig-core/#sec-X509Data)
  *
  *************************************************************************/
 XMLSEC_KEY_DATA_DECLARE(MSCryptoX509Data, xmlSecMSCryptoX509DataCtx)
@@ -124,11 +127,11 @@ typedef struct _xmlSecMSCryptoKeyDataX509Context {
 } xmlSecMSCryptoKeyDataX509Context;
 
 static int              xmlSecMSCryptoKeyDataX509Read          (xmlSecKeyDataPtr data,
-                                                                xmlSecKeyValueX509Ptr x509Value,
+                                                                xmlSecKeyX509DataValuePtr x509Value,
                                                                 xmlSecKeysMngrPtr keysMngr,
                                                                 unsigned int flags);
 static int              xmlSecMSCryptoKeyDataX509Write         (xmlSecKeyDataPtr data,
-                                                                xmlSecKeyValueX509Ptr x509Value,
+                                                                xmlSecKeyX509DataValuePtr x509Value,
                                                                 int content,
                                                                 void* context);
 
@@ -138,7 +141,7 @@ static xmlSecKeyDataKlass xmlSecMSCryptoKeyDataX509Klass = {
 
     /* data */
     xmlSecNameX509Data,
-    xmlSecKeyDataUsageKeyInfoNode | xmlSecKeyDataUsageRetrievalMethodNodeXml,
+    xmlSecKeyDataUsageReadFromFile | xmlSecKeyDataUsageKeyInfoNode | xmlSecKeyDataUsageRetrievalMethodNodeXml,
                                                 /* xmlSecKeyDataUsage usage; */
     xmlSecHrefX509Data,                         /* const xmlChar* href; */
     xmlSecNodeX509Data,                         /* const xmlChar* dataNodeName; */
@@ -540,23 +543,19 @@ xmlSecMSCryptoKeyDataX509XmlRead(xmlSecKeyDataId id, xmlSecKeyPtr key,
 
     data = xmlSecKeyEnsureData(key, id);
     if (data == NULL) {
-        xmlSecInternalError("xmlSecKeyEnsureData",
-            xmlSecKeyDataKlassGetName(id));
+        xmlSecInternalError("xmlSecKeyEnsureData", xmlSecKeyDataKlassGetName(id));
         return(-1);
     }
 
-    ret = xmlSecKeyDataX509XmlRead(data, node, keyInfoCtx,
-        xmlSecMSCryptoKeyDataX509Read);
+    ret = xmlSecKeyDataX509XmlRead(key, data, node, keyInfoCtx, xmlSecMSCryptoKeyDataX509Read);
     if (ret < 0) {
-        xmlSecInternalError("xmlSecKeyDataX509XmlRead",
-            xmlSecKeyDataKlassGetName(id));
+        xmlSecInternalError("xmlSecKeyDataX509XmlRead", xmlSecKeyDataKlassGetName(id));
         return(-1);
     }
 
     ret = xmlSecMSCryptoKeyDataX509VerifyAndExtractKey(data, key, keyInfoCtx);
     if (ret < 0) {
-        xmlSecInternalError("xmlSecMSCryptoKeyDataX509VerifyAndExtractKey",
-            xmlSecKeyDataKlassGetName(id));
+        xmlSecInternalError("xmlSecMSCryptoKeyDataX509VerifyAndExtractKey", xmlSecKeyDataKlassGetName(id));
         return(-1);
     }
     return(0);
@@ -680,9 +679,9 @@ xmlSecMSCryptoKeyDataX509DebugXmlDump(xmlSecKeyDataPtr data, FILE* output) {
     fprintf(output, "</X509Data>\n");
 }
 
-
+/* xmlSecKeyDataX509Read: 0 on success and a negative value otherwise */
 static int
-xmlSecMSCryptoKeyDataX509Read(xmlSecKeyDataPtr data, xmlSecKeyValueX509Ptr x509Value,
+xmlSecMSCryptoKeyDataX509Read(xmlSecKeyDataPtr data, xmlSecKeyX509DataValuePtr x509Value,
     xmlSecKeysMngrPtr keysMngr, unsigned int flags) {
     xmlSecKeyDataStorePtr x509Store;
     int stopOnUnknownCert = 0;
@@ -787,9 +786,11 @@ done:
     return(res);
 }
 
-
+/* xmlSecKeyDataX509Write: returns 1 on success, 0 if no more certs/crls are available,
+ * or a negative value if an error occurs.
+ */
 static int
-xmlSecMSCryptoKeyDataX509Write(xmlSecKeyDataPtr data, xmlSecKeyValueX509Ptr x509Value,
+xmlSecMSCryptoKeyDataX509Write(xmlSecKeyDataPtr data, xmlSecKeyX509DataValuePtr x509Value,
     int content, void* context) {
     xmlSecMSCryptoKeyDataX509Context* ctx;
     int ret;
@@ -809,7 +810,7 @@ xmlSecMSCryptoKeyDataX509Write(xmlSecKeyDataPtr data, xmlSecKeyValueX509Ptr x509
                 "pos=" XMLSEC_SIZE_FMT, ctx->crtPos);
             return(-1);
         }
-        if ((content & XMLSEC_X509DATA_CERTIFICATE_NODE) != 0) {
+        if (XMLSEC_X509DATA_HAS_EMPTY_NODE(content, XMLSEC_X509DATA_CERTIFICATE_NODE)) {
             xmlSecAssert2(cert->pbCertEncoded != NULL, -1);
             xmlSecAssert2(cert->cbCertEncoded > 0, -1);
 
@@ -822,7 +823,7 @@ xmlSecMSCryptoKeyDataX509Write(xmlSecKeyDataPtr data, xmlSecKeyValueX509Ptr x509
                 return(-1);
             }
         }
-        if ((content & XMLSEC_X509DATA_SKI_NODE) != 0) {
+        if (XMLSEC_X509DATA_HAS_EMPTY_NODE(content, XMLSEC_X509DATA_SKI_NODE)) {
             ret = xmlSecMSCryptoX509SKIWrite(cert, &(x509Value->ski));
             if (ret < 0) {
                 xmlSecInternalError2("xmlSecMSCryptoX509SKIWrite",
@@ -831,7 +832,7 @@ xmlSecMSCryptoKeyDataX509Write(xmlSecKeyDataPtr data, xmlSecKeyValueX509Ptr x509
                 return(-1);
             }
         }
-        if ((content & XMLSEC_X509DATA_SUBJECTNAME_NODE) != 0) {
+        if (XMLSEC_X509DATA_HAS_EMPTY_NODE(content, XMLSEC_X509DATA_SUBJECTNAME_NODE)) {
             xmlSecAssert2(x509Value->subject == NULL, -1);
             xmlSecAssert2(cert->pCertInfo != NULL, -1);
 
@@ -843,7 +844,7 @@ xmlSecMSCryptoKeyDataX509Write(xmlSecKeyDataPtr data, xmlSecKeyValueX509Ptr x509
                 return(-1);
             }
         }
-        if ((content & XMLSEC_X509DATA_ISSUERSERIAL_NODE) != 0) {
+        if (XMLSEC_X509DATA_HAS_EMPTY_NODE(content, XMLSEC_X509DATA_ISSUERSERIAL_NODE)) {
             xmlSecAssert2(x509Value->issuerName == NULL, -1);
             xmlSecAssert2(x509Value->issuerSerial == NULL, -1);
             xmlSecAssert2(cert->pCertInfo != NULL, -1);
@@ -889,11 +890,11 @@ xmlSecMSCryptoKeyDataX509Write(xmlSecKeyDataPtr data, xmlSecKeyValueX509Ptr x509
     }
     else {
         /* no more certs or crls */
-        return(1);
+        return(0);
     }
 
     /* success */
-    return(0);
+    return(1);
 }
 
 static int
