@@ -1,18 +1,16 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
+ * X509 certificates verification support functions for Microsoft Crypto API.
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
  * Copyright (C) 2003 Cordys R&D BV, All rights reserved.
- * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 /**
- * SECTION:x509vfy
- * @Short_description: X509 certificates verification support functions for Microsoft Crypto API.
- * @Stability: Private
- *
+ * SECTION:x509
  */
 
 #include "globals.h"
@@ -106,7 +104,7 @@ xmlSecMSCryptoX509StoreGetKlass(void) {
  * @issuerName:     the desired certificate issuer name.
  * @issuerSerial:   the desired certificate issuer serial number.
  * @ski:            the desired certificate SKI.
- * @keyInfoCtx:     the pointer to <dsig:KeyInfo/> element processing context.
+ * @keyInfoCtx:     the pointer to &lt;dsig:KeyInfo/&gt; element processing context.
  *
  * Searches @store for a certificate that matches given criteria.
  *
@@ -147,7 +145,7 @@ xmlSecMSCryptoX509StoreFindCert(xmlSecKeyDataStorePtr store, xmlChar *subjectNam
  * @issuerSerial:   the desired certificate issuer serial number.
  * @ski:            the desired certificate SKI.
  * @skiSize:        the desired certificate SKI size.
- * @keyInfoCtx:     the pointer to <dsig:KeyInfo/> element processing context.
+ * @keyInfoCtx:     the pointer to &lt;dsig:KeyInfo/&gt; element processing context.
  *
  * Searches @store for a certificate that matches given criteria.
  *
@@ -235,6 +233,53 @@ xmlSecMSCryptoCheckRevocation(HCERTSTORE hStore, PCCERT_CONTEXT pCert) {
     return(TRUE);
 }
 
+static void
+xmlSecMSCryptoX509StoreCertError(xmlSecKeyDataStorePtr store, PCCERT_CONTEXT cert, DWORD flags) {
+    xmlChar * subject = NULL;
+
+    xmlSecAssert(xmlSecKeyDataStoreCheckId(store, xmlSecMSCryptoX509StoreId));
+    xmlSecAssert(cert != NULL);
+    xmlSecAssert(flags != 0);
+
+    /* get certs subject */
+    subject = xmlSecMSCryptoX509GetNameString(cert, CERT_NAME_RDN_TYPE, 0, NULL);
+    if(subject == NULL) {
+        xmlSecInternalError("xmlSecMSCryptoX509GetNameString", NULL);
+        return;
+    }
+
+    /* print error */
+    if (flags & CERT_STORE_SIGNATURE_FLAG) {
+        xmlSecOtherError2(XMLSEC_ERRORS_R_CERT_VERIFY_FAILED,
+                          xmlSecKeyDataStoreGetName(store),
+                          "signature failed, subject=%s",
+                          xmlSecErrorsSafeString(subject));
+    } else if (flags & CERT_STORE_TIME_VALIDITY_FLAG) {
+        xmlSecOtherError2(XMLSEC_ERRORS_R_CERT_HAS_EXPIRED,
+                          xmlSecKeyDataStoreGetName(store),
+                          "subject=%s",
+                          xmlSecErrorsSafeString(subject));
+    } else if (flags & CERT_STORE_REVOCATION_FLAG) {
+        if (flags & CERT_STORE_NO_CRL_FLAG) {
+            xmlSecOtherError2(XMLSEC_ERRORS_R_CERT_REVOKED,
+                              xmlSecKeyDataStoreGetName(store),
+                              "no crl, subject=%s",
+                              xmlSecErrorsSafeString(subject));
+        } else {
+            xmlSecOtherError2(XMLSEC_ERRORS_R_CERT_REVOKED,
+                              xmlSecKeyDataStoreGetName(store),
+                              "subject=%s",
+                              xmlSecErrorsSafeString(subject));
+        }
+    } else {
+        xmlSecOtherError2(XMLSEC_ERRORS_R_CERT_VERIFY_FAILED,
+                          xmlSecKeyDataStoreGetName(store),
+                          "subject=%s",
+                          xmlSecErrorsSafeString(subject));
+    }
+
+    xmlFree(subject);
+}
 
 /**
  * xmlSecBuildChainUsingWinapi:
@@ -313,87 +358,6 @@ end:
     return (rc);
 }
 
-
-
-/* this function does NOT check for time validity (see xmlSecMSCngVerifyCertTime)
-*  returns <0 if there is an error; 0 if verification failed and >0 if verification succeeded */
-static int
-xmlSecMSCryptoX509StoreVerifySubject(xmlSecKeyDataStorePtr store, PCCERT_CONTEXT cert, PCCERT_CONTEXT issuerCert) {
-    DWORD flags;
-    BOOL ret;
-
-    xmlSecAssert2(xmlSecKeyDataStoreCheckId(store, xmlSecMSCryptoX509StoreId), -1);
-    xmlSecAssert2(cert != NULL, -1);
-    xmlSecAssert2(issuerCert != NULL, -1);
-
-    flags = CERT_STORE_REVOCATION_FLAG | CERT_STORE_SIGNATURE_FLAG;
-    ret = CertVerifySubjectCertificateContext(cert, issuerCert, &flags);
-    if (!ret) {
-        xmlSecMSCryptoError("CertVerifySubjectCertificateContext", NULL);
-        return(-1);
-    }
-
-    /* parse returned flags: https://learn.microsoft.com/en-us/previous-versions/windows/embedded/ms883939(v=msdn.10) */
-    if ((flags & CERT_STORE_SIGNATURE_FLAG) != 0) {
-        xmlSecOtherError(XMLSEC_ERRORS_R_CERT_VERIFY_FAILED,
-            xmlSecKeyDataStoreGetName(store),
-            "CertVerifySubjectCertificateContext: CERT_STORE_SIGNATURE_FLAG");
-        return(0);
-    }
-    else if (((flags & CERT_STORE_REVOCATION_FLAG) != 0) && ((flags & CERT_STORE_NO_CRL_FLAG) == 0)) {
-        /* If CERT_STORE_REVOCATION_FLAG is enabled and the issuer does not have a CRL in the store,
-        then CERT_STORE_NO_CRL_FLAG is set in addition to CERT_STORE_REVOCATION_FLAG. */
-        xmlSecOtherError(XMLSEC_ERRORS_R_CERT_VERIFY_FAILED,
-            xmlSecKeyDataStoreGetName(store),
-            "CertVerifySubjectCertificateContext: CERT_STORE_REVOCATION_FLAG");
-        return(0);
-    }
-
-    /* success */
-    return(1);
-}
-
-static int
-xmlSecMSCryptoX509StoreContainsCert(HCERTSTORE store, CERT_NAME_BLOB* name,
-    PCCERT_CONTEXT cert, xmlSecKeyDataStorePtr keyDataStore)
-{
-    PCCERT_CONTEXT storeCert = NULL;
-    int ret;
-
-    xmlSecAssert2(store != NULL, -1);
-    xmlSecAssert2(name != NULL, -1);
-    xmlSecAssert2(cert != NULL, -1);
-    xmlSecAssert2(keyDataStore != NULL, -1);
-
-    storeCert = CertFindCertificateInStore(store,
-        X509_ASN_ENCODING | PKCS_7_ASN_ENCODING,
-        0,
-        CERT_FIND_SUBJECT_NAME,
-        name,
-        NULL);
-    if (storeCert == NULL) {
-        return (0);
-    }
-
-    ret = xmlSecMSCryptoX509StoreVerifySubject(keyDataStore, cert, storeCert);
-    if (ret < 0) {
-        xmlSecInternalError("xmlSecMSCryptoX509StoreVerifySubject", NULL);
-        CertFreeCertificateContext(storeCert);
-        return(-1);
-    } else if (ret == 0) {
-        xmlSecOtherError(XMLSEC_ERRORS_R_CERT_VERIFY_FAILED,
-            NULL,
-            "xmlSecMSCryptoX509StoreVerifySubject");
-        CertFreeCertificateContext(storeCert);
-        return(-1);
-    }
-
-    /* success */
-    CertFreeCertificateContext(storeCert);
-    return(1);
-}
-
-
 /**
  * xmlSecMSCryptoBuildCertChainManually:
  * @cert: the certificate we check
@@ -412,53 +376,59 @@ xmlSecMSCryptoBuildCertChainManually (PCCERT_CONTEXT cert, LPFILETIME pfTime,
         HCERTSTORE store_trusted, HCERTSTORE store_untrusted, HCERTSTORE certs,
         xmlSecKeyDataStorePtr store) {
     PCCERT_CONTEXT issuerCert = NULL;
-    int ret;
+    DWORD flags;
 
-    /* check certificate validity and revokation */
     if (!xmlSecMSCryptoVerifyCertTime(cert, pfTime)) {
-        xmlSecOtherError(XMLSEC_ERRORS_R_CERT_HAS_EXPIRED,
-            xmlSecKeyDataStoreGetName(store),
-            "certificate expired");
+        xmlSecMSCryptoX509StoreCertError(store, cert, CERT_STORE_TIME_VALIDITY_FLAG);
         return(FALSE);
     }
 
     if (!xmlSecMSCryptoCheckRevocation(certs, cert)) {
-        xmlSecOtherError(XMLSEC_ERRORS_R_CRL_VERIFY_FAILED,
-            xmlSecKeyDataStoreGetName(store),
-            "certificate revoked");;
         return(FALSE);
     }
 
-    /* does trustedStore contain cert directly? */
-    ret = xmlSecMSCryptoX509StoreContainsCert(store_trusted,
-        &(cert->pCertInfo->Subject), cert, store);
-    if (ret < 0) {
-        xmlSecInternalError("xmlSecMSCryptoX509StoreContainsCert", NULL);
+    /*
+     * Try to find the cert in the trusted cert store. We will trust
+     * the certificate in the trusted store.
+     */
+    issuerCert = CertFindCertificateInStore(store_trusted,
+                X509_ASN_ENCODING | PKCS_7_ASN_ENCODING,
+                0,
+                CERT_FIND_SUBJECT_NAME,
+                &(cert->pCertInfo->Subject),
+                NULL);
+    if( issuerCert != NULL) {
+        /* We have found the trusted cert, so return true */
+        /* todo: do we want to verify the trusted cert's revocation? we must, I think */
+        CertFreeCertificateContext( issuerCert ) ;
+        return( TRUE ) ;
+    }
+
+    /* Check whether the certificate is self signed certificate */
+    if(CertCompareCertificateName(X509_ASN_ENCODING | PKCS_7_ASN_ENCODING, &(cert->pCertInfo->Subject), &(cert->pCertInfo->Issuer))) {
         return(FALSE);
-    } else if (ret == 1) {
-        /* success */
+    }
+
+    /* try to find issuer cert in the trusted cert in the store */
+    issuerCert = CertFindCertificateInStore(store_trusted,
+                X509_ASN_ENCODING | PKCS_7_ASN_ENCODING,
+                0,
+                CERT_FIND_SUBJECT_NAME,
+                &(cert->pCertInfo->Issuer),
+                NULL);
+    if(issuerCert != NULL) {
+        flags = CERT_STORE_REVOCATION_FLAG | CERT_STORE_SIGNATURE_FLAG;
+        if(!CertVerifySubjectCertificateContext(cert, issuerCert, &flags)) {
+            xmlSecMSCryptoX509StoreCertError(store, issuerCert, flags);
+            CertFreeCertificateContext(issuerCert);
+            return(FALSE);
+        }
+            /* todo: do we want to verify the trusted cert? we must check
+                 * revocation, I think */
+        CertFreeCertificateContext(issuerCert);
         return(TRUE);
     }
 
-    /* does trustedStore contain the issuer cert? */
-    ret = xmlSecMSCryptoX509StoreContainsCert(store_trusted,
-        &(cert->pCertInfo->Issuer), cert, store);
-    if (ret < 0) {
-        xmlSecInternalError("xmlSecMSCryptoX509StoreContainsCert", NULL);
-        return(FALSE);
-    } else if (ret == 1) {
-        /* success */
-        return(TRUE);
-    }
-
-    /* is cert self-signed? no recursion in that case */
-    if (CertCompareCertificateName(X509_ASN_ENCODING | PKCS_7_ASN_ENCODING,
-        &(cert->pCertInfo->Subject),
-        &(cert->pCertInfo->Issuer))) {
-        /* not verified */
-        return(FALSE);
-    }
-
     /* try the untrusted certs in the chain */
     issuerCert = CertFindCertificateInStore(certs,
                 X509_ASN_ENCODING | PKCS_7_ASN_ENCODING,
@@ -467,27 +437,17 @@ xmlSecMSCryptoBuildCertChainManually (PCCERT_CONTEXT cert, LPFILETIME pfTime,
                 &(cert->pCertInfo->Issuer),
                 NULL);
     if(issuerCert != NULL) {
-        ret = xmlSecMSCryptoX509StoreVerifySubject(store, cert, issuerCert);
-        if (ret < 0) {
-            xmlSecInternalError("xmlSecMSCryptoX509StoreVerifySubject", NULL);
+        flags = CERT_STORE_REVOCATION_FLAG | CERT_STORE_SIGNATURE_FLAG;
+        if(!CertVerifySubjectCertificateContext(cert, issuerCert, &flags)) {
+            xmlSecMSCryptoX509StoreCertError(store, issuerCert, flags);
             CertFreeCertificateContext(issuerCert);
             return(FALSE);
         }
-        else if (ret == 0) {
-            xmlSecOtherError(XMLSEC_ERRORS_R_CERT_VERIFY_FAILED,
-                NULL,
-                "xmlSecMSCryptoX509StoreVerifySubject");
+        if(!xmlSecMSCryptoBuildCertChainManually(issuerCert, pfTime, store_trusted, store_untrusted, certs, store)) {
+            xmlSecMSCryptoX509StoreCertError(store, issuerCert, flags);
             CertFreeCertificateContext(issuerCert);
             return(FALSE);
         }
-
-        if (!xmlSecMSCryptoBuildCertChainManually(issuerCert, pfTime, store_trusted, store_untrusted, certs, store)) {
-            xmlSecInternalError("xmlSecMSCryptoBuildCertChainManually", NULL);
-            CertFreeCertificateContext(issuerCert);
-            return(FALSE);
-        }
-
-        /* success */
         CertFreeCertificateContext(issuerCert);
         return(TRUE);
     }
@@ -500,32 +460,20 @@ xmlSecMSCryptoBuildCertChainManually (PCCERT_CONTEXT cert, LPFILETIME pfTime,
                 &(cert->pCertInfo->Issuer),
                 NULL);
     if(issuerCert != NULL) {
-        ret = xmlSecMSCryptoX509StoreVerifySubject(store, cert, issuerCert);
-        if (ret < 0) {
-            xmlSecInternalError("xmlSecMSCryptoX509StoreVerifySubject", NULL);
+        flags = CERT_STORE_REVOCATION_FLAG | CERT_STORE_SIGNATURE_FLAG;
+        if(!CertVerifySubjectCertificateContext(cert, issuerCert, &flags)) {
+            xmlSecMSCryptoX509StoreCertError(store, issuerCert, flags);
             CertFreeCertificateContext(issuerCert);
             return(FALSE);
         }
-        else if (ret == 0) {
-            xmlSecOtherError(XMLSEC_ERRORS_R_CERT_VERIFY_FAILED,
-                NULL,
-                "xmlSecMSCryptoX509StoreVerifySubject");
+        if(!xmlSecMSCryptoBuildCertChainManually(issuerCert, pfTime, store_trusted, store_untrusted, certs, store)) {
             CertFreeCertificateContext(issuerCert);
             return(FALSE);
         }
-
-        if (!xmlSecMSCryptoBuildCertChainManually(issuerCert, pfTime, store_trusted, store_untrusted, certs, store)) {
-            xmlSecInternalError("xmlSecMSCryptoBuildCertChainManually", NULL);
-            CertFreeCertificateContext(issuerCert);
-            return(FALSE);
-        }
-
-        /* success */
         CertFreeCertificateContext(issuerCert);
         return(TRUE);
     }
 
-    /* no luck */
     return(FALSE);
 }
 
@@ -578,7 +526,7 @@ xmlSecMSCryptoX509StoreConstructCertsChain(xmlSecKeyDataStorePtr store, PCCERT_C
  * xmlSecMSCryptoX509StoreVerify:
  * @store:        the pointer to X509 certificate context store klass.
  * @certs:        the untrusted certificates stack.
- * @keyInfoCtx:        the pointer to <dsig:KeyInfo/> element processing context.
+ * @keyInfoCtx:        the pointer to &lt;dsig:KeyInfo/&gt; element processing context.
  *
  * Verifies @certs list.
  *
@@ -1361,5 +1309,3 @@ xmlSecMSCryptoX509GetNameString(PCCERT_CONTEXT pCertContext, DWORD dwType, DWORD
 }
 
 #endif /* XMLSEC_NO_X509 */
-
-
