@@ -1,6 +1,7 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
+ * X509 certificates verification support functions for NSS.
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
@@ -8,10 +9,7 @@
  * Copyright (c) 2003 America Online, Inc.  All rights reserved.
  */
 /**
- * SECTION:x509vfy
- * @Short_description: X509 certificates verification support functions for NSS.
- * @Stability: Private
- *
+ * SECTION:x509
  */
 
 #include "globals.h"
@@ -26,6 +24,8 @@
 
 #include <cert.h>
 #include <secerr.h>
+#include <secder.h>
+#include <sechash.h>
 
 #include <xmlsec/xmlsec.h>
 #include <xmlsec/keys.h>
@@ -40,6 +40,7 @@
 #include <xmlsec/nss/x509.h>
 
 #include "../cast_helpers.h"
+#include "private.h"
 
 /**************************************************************************
  *
@@ -59,6 +60,9 @@ struct _xmlSecNssX509StoreCtx {
      */
 
     CERTCertList* certsList; /* just keeping a reference to destroy later */
+
+    xmlSecNssX509CrlNodePtr crlsList;
+    unsigned int     numCrls;
 };
 
 /****************************************************************************
@@ -80,7 +84,8 @@ static int              xmlSecNssX509NameStringRead     (const xmlSecByte **in,
                                                          int ingoreTrailingSpaces);
 static xmlSecByte *     xmlSecNssX509NameRead           (const xmlChar *str);
 
-static int              xmlSecNssNumToItem              (SECItem *it,
+static int              xmlSecNssNumToItem              (PLArenaPool *arena,
+                                                         SECItem *it,
                                                          PRUint64 num);
 
 
@@ -100,12 +105,7 @@ static xmlSecKeyDataStoreKlass xmlSecNssX509StoreKlass = {
     NULL,                                       /* void* reserved1; */
 };
 
-static CERTCertificate*         xmlSecNssX509FindCert(CERTCertList* certsList,
-                                                      const xmlChar *subjectName,
-                                                      const xmlChar *issuerName,
-                                                      const xmlChar *issuerSerial,
-                                                      xmlSecByte * ski,
-                                                      xmlSecSize skiSize);
+static CERTCertificate*         xmlSecNssX509FindCert(CERTCertList* certsList, xmlSecNssX509FindCertCtxPtr findCertCtx);
 
 
 /**
@@ -127,7 +127,7 @@ xmlSecNssX509StoreGetKlass(void) {
  * @issuerName:         the desired certificate issuer name.
  * @issuerSerial:       the desired certificate issuer serial number.
  * @ski:                the desired certificate SKI.
- * @keyInfoCtx:         the pointer to <dsig:KeyInfo/> element processing context.
+ * @keyInfoCtx:         the pointer to &lt;dsig:KeyInfo/&gt; element processing context.
  *
  * Searches @store for a certificate that matches given criteria.
  *
@@ -168,9 +168,9 @@ xmlSecNssX509StoreFindCert(xmlSecKeyDataStorePtr store, xmlChar *subjectName,
  * @issuerSerial:       the desired certificate issuer serial number.
  * @ski:                the desired certificate SKI.
  * @skiSize:            the desired certificate SKI size.
- * @keyInfoCtx:         the pointer to <dsig:KeyInfo/> element processing context.
+ * @keyInfoCtx:         the pointer to &lt;dsig:KeyInfo/&gt; element processing context.
  *
- * Searches @store for a certificate that matches given criteria.
+ * Deprecated. Searches @store for a certificate that matches given criteria.
  *
  * Returns: pointer to found certificate or NULL if certificate is not found
  * or an error occurs.
@@ -181,41 +181,265 @@ xmlSecNssX509StoreFindCert_ex(xmlSecKeyDataStorePtr store, xmlChar *subjectName,
                                  xmlSecByte * ski, xmlSecSize skiSize,
                                  xmlSecKeyInfoCtx* keyInfoCtx ATTRIBUTE_UNUSED) {
     xmlSecNssX509StoreCtxPtr ctx;
+    xmlSecNssX509FindCertCtx findCertCtx;
+    CERTCertificate * cert;
+    int ret;
 
+    xmlSecAssert2(store != NULL, NULL);
     xmlSecAssert2(xmlSecKeyDataStoreCheckId(store, xmlSecNssX509StoreId), NULL);
     UNREFERENCED_PARAMETER(keyInfoCtx);
 
     ctx = xmlSecNssX509StoreGetCtx(store);
     xmlSecAssert2(ctx != NULL, NULL);
 
-    return xmlSecNssX509FindCert(ctx->certsList, subjectName,
-        issuerName, issuerSerial,
-        ski, skiSize);
+    /* do we have any certs to look through? */
+    if(ctx->certsList == NULL) {
+        return(NULL);
+    }
+
+    ret = xmlSecNssX509FindCertCtxInitialize(&findCertCtx,
+            subjectName,
+            issuerName, issuerSerial,
+            ski, skiSize);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecNssX509FindCertCtxInitialize", NULL);
+        xmlSecNssX509FindCertCtxFinalize(&findCertCtx);
+        return(NULL);
+    }
+
+    cert = xmlSecNssX509FindCert(ctx->certsList, &findCertCtx);
+
+    /* done */
+    xmlSecNssX509FindCertCtxFinalize(&findCertCtx);
+    return(cert);
 }
 
+CERTCertificate *
+xmlSecNssX509StoreFindCertByValue(xmlSecKeyDataStorePtr store, xmlSecKeyX509DataValuePtr x509Value) {
+    xmlSecNssX509StoreCtxPtr ctx;
+    xmlSecNssX509FindCertCtx findCertCtx;
+    CERTCertificate * cert;
+    int ret;
+
+    xmlSecAssert2(store != NULL, NULL);
+    xmlSecAssert2(xmlSecKeyDataStoreCheckId(store, xmlSecNssX509StoreId), NULL);
+    xmlSecAssert2(x509Value != NULL, NULL);
+
+    ctx = xmlSecNssX509StoreGetCtx(store);
+    xmlSecAssert2(ctx != NULL, NULL);
+
+    /* do we have any certs to look through? */
+    if(ctx->certsList == NULL) {
+        return(NULL);
+    }
+
+
+    ret = xmlSecNssX509FindCertCtxInitializeFromValue(&findCertCtx, x509Value);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecNssX509FindCertCtxInitializeFromValue", NULL);
+        xmlSecNssX509FindCertCtxFinalize(&findCertCtx);
+        return(NULL);
+    }
+
+    cert = xmlSecNssX509FindCert(ctx->certsList, &findCertCtx);
+
+    /* done */
+    xmlSecNssX509FindCertCtxFinalize(&findCertCtx);
+    return(cert);
+}
+
+/* returns 1 if cert was revoked, 0 if not, and a negative value if an error occurs */
+static int
+xmlSecNssX509StoreCheckIfCertIsRevoked(CERTCertificate* cert, CERTSignedCrl* crl, xmlSecKeyInfoCtx* keyInfoCtx) {
+    CERTCrlEntry *entry;
+    SECStatus rv;
+    int ret;
+    int ii;
+
+    xmlSecAssert2(cert != NULL, -1);
+    xmlSecAssert2(crl != NULL, -1);
+    xmlSecAssert2(keyInfoCtx != NULL, -1);
+
+    /* do we have any revocation entries? */
+    if (crl->crl.entries == NULL) {
+        return(0);
+    }
+    for(ii = 0; ((entry = crl->crl.entries[ii]) != NULL); ++ii) {
+        if (SECITEM_CompareItem(&(cert->serialNumber), &(entry->serialNumber)) != SECEqual) {
+            continue;
+        }
+        /* check revocation date: if we are checking against current time, we assume
+         * that CRL and revocation do NOT come from the future and we don't need to check
+         * the timestamps */
+        if(keyInfoCtx->certsVerificationTime > 0) {
+            PRTime revocationDate = 0;
+            time_t revocationTs = 0;
+
+            rv = DER_DecodeTimeChoice(&revocationDate, &(entry->revocationDate));
+            if((rv != SECSuccess) || (revocationDate == 0)) {
+                xmlSecNssError("DER_DecodeTimeChoice(revocationDate)", NULL);
+                return(-1);
+            }
+            ret = xmlSecNssX509CertGetTime(&revocationDate, &revocationTs);
+            if((ret < 0) || (revocationTs == 0)) {
+                xmlSecInternalError("xmlSecNssX509CertGetTime(revocationDate)", NULL);
+                return(-1);
+            }
+            if(keyInfoCtx->certsVerificationTime < revocationTs) {
+                /* verification time before revocation ts, this doesn't apply */
+                continue;
+            }
+        }
+
+        /* we found a valid revocation entry for this cert */
+        return(1);
+    }
+
+    /* not found */
+    return(0);
+}
+
+static int
+xmlSecNssX509StoreFindBestCrl(xmlSecNssX509StoreCtxPtr x509StoreCtx, CERTCertificate* cert, CERTSignedCrl ** res, xmlSecKeyInfoCtx* keyInfoCtx) {
+    xmlSecNssX509CrlNodePtr cur;
+    PRTime lastUpdate = 0;
+    PRTime resLastUpdate = 0;
+    SECStatus rv;
+
+    xmlSecAssert2(x509StoreCtx != NULL, -1);
+    xmlSecAssert2(cert != NULL, -1);
+    xmlSecAssert2(res != NULL, -1);
+    xmlSecAssert2((*res) == NULL, -1);
+    xmlSecAssert2(keyInfoCtx != NULL, -1);
+
+    /* find best matching CRL */
+    for(cur = x509StoreCtx->crlsList; cur != NULL; cur = cur->next) {
+        if(cur->crl == NULL) {
+            continue;
+        }
+        if (SECITEM_CompareItem(&(cert->derIssuer), &(cur->crl->crl.derName)) != SECEqual) {
+            continue;
+        }
+
+        /* get lastUpdate time */
+        rv = DER_DecodeTimeChoice(&lastUpdate, &(cur->crl->crl.lastUpdate));
+        if((rv != SECSuccess) || (lastUpdate == 0)) {
+            xmlSecNssError("DER_DecodeTimeChoice(lastUpdate)", NULL);
+            return(-1);
+        }
+
+        /* Use latest CRL by the last update time */
+        if(((*res) == NULL) || (resLastUpdate < lastUpdate)) {
+            (*res) = cur->crl;
+            resLastUpdate = lastUpdate;
+        }
+    }
+
+    /* done! */
+    return(0);
+}
+
+static int
+xmlSecNssX509StoreRemoveRevokedCerts(xmlSecNssX509StoreCtxPtr x509StoreCtx, CERTCertList* certs,
+    CERTCertList** res, xmlSecKeyInfoCtx* keyInfoCtx
+) {
+    CERTCertListNode* cur;
+    CERTCertificate* cert;
+    SECStatus rv;
+    int ret;
+
+    xmlSecAssert2(x509StoreCtx != NULL, -1);
+    xmlSecAssert2(certs != NULL, -1);
+    xmlSecAssert2(res != NULL, -1);
+    xmlSecAssert2((*res) == NULL, -1);
+    xmlSecAssert2(keyInfoCtx != NULL, -1);
+
+    (*res) = CERT_NewCertList();
+    if((*res) == NULL) {
+        xmlSecNssError("CERT_NewCertList", NULL);
+        return(-1);
+    }
+
+    for (cur = CERT_LIST_HEAD(certs); !CERT_LIST_END(cur, certs); cur = CERT_LIST_NEXT(cur)) {
+        CERTSignedCrl* crl = NULL;
+
+        if(cur->cert == NULL) {
+            continue;
+        }
+
+        ret = xmlSecNssX509StoreFindBestCrl(x509StoreCtx, cur->cert, &crl, keyInfoCtx);
+        if(ret < 0) {
+            xmlSecInternalError("xmlSecNssX509StoreFindBestCrl",  NULL);
+            return(-1);
+        }
+        if(crl != NULL) {
+            ret = xmlSecNssX509StoreCheckIfCertIsRevoked(cur->cert, crl, keyInfoCtx);
+            if(ret < 0) {
+                xmlSecInternalError("xmlSecNssX509StoreFindBestCrl",  NULL);
+                return(-1);
+            } else if(ret != 0) {
+                /* cert was revoked */
+                continue;
+            }
+        }
+
+        /* add to the output */
+        cert = CERT_DupCertificate(cur->cert);
+        if(cert == NULL) {
+            xmlSecNssError("CERT_DupCertificate", NULL);
+            return(-1);
+        }
+        rv = CERT_AddCertToListTail((*res),  cert);
+        if(rv != SECSuccess) {
+            xmlSecNssError("CERT_AddCertToListTail", NULL);
+            return(-1);
+        }
+    }
+
+    /* done */
+    return(0);
+}
+
+static CERTCertificate *
+xmlSecNssX509StoreFindChildCert(CERTCertificate* cert, CERTCertList* certs) {
+    CERTCertListNode* cur;
+
+    xmlSecAssert2(cert != NULL, NULL);
+    xmlSecAssert2(certs != NULL, NULL);
+
+     for (cur = CERT_LIST_HEAD(certs); !CERT_LIST_END(cur, certs); cur = CERT_LIST_NEXT(cur)) {
+        /* allow self signed certs */
+        if(cur->cert == cert) {
+            continue;
+        }
+        if (SECITEM_CompareItem(&(cur->cert->derIssuer), &(cert->derSubject)) == SECEqual) {
+            return(cur->cert);
+        }
+     }
+     return(NULL);
+}
 
 /**
  * xmlSecNssX509StoreVerify:
  * @store:              the pointer to X509 key data store klass.
  * @certs:              the untrusted certificates stack.
- * @keyInfoCtx:         the pointer to <dsig:KeyInfo/> element processing context.
+ * @keyInfoCtx:         the pointer to &lt;dsig:KeyInfo/&gt; element processing context.
  *
  * Verifies @certs list.
  *
  * Returns: pointer to the first verified certificate from @certs.
  */
 CERTCertificate *
-xmlSecNssX509StoreVerify(xmlSecKeyDataStorePtr store, CERTCertList* certs,
-                         xmlSecKeyInfoCtx* keyInfoCtx) {
+xmlSecNssX509StoreVerify(xmlSecKeyDataStorePtr store, CERTCertList* certs, xmlSecKeyInfoCtx* keyInfoCtx) {
     xmlSecNssX509StoreCtxPtr ctx;
-    CERTCertListNode*       head;
-    CERTCertificate*       cert = NULL;
-    CERTCertListNode*       head1;
-    CERTCertificate*       cert1 = NULL;
+    CERTCertListNode* cur;
+    CERTCertList* good_certs = NULL;
+    CERTCertificate* cert = NULL;
     SECStatus status = SECFailure;
     int64 timeboundary;
     int64 tmp1, tmp2;
     PRErrorCode err;
+    int ret;
 
     xmlSecAssert2(xmlSecKeyDataStoreCheckId(store, xmlSecNssX509StoreId), NULL);
     xmlSecAssert2(certs != NULL, NULL);
@@ -234,48 +458,53 @@ xmlSecNssX509StoreVerify(xmlSecKeyDataStorePtr store, CERTCertList* certs,
         timeboundary = PR_Now();
     }
 
-    for (head = CERT_LIST_HEAD(certs);
-         !CERT_LIST_END(head, certs);
-         head = CERT_LIST_NEXT(head)) {
-        cert = head->cert;
-
-        /* if cert is the issuer of any other cert in the list, then it is
-         * to be skipped */
-        for (head1 = CERT_LIST_HEAD(certs);
-             !CERT_LIST_END(head1, certs);
-             head1 = CERT_LIST_NEXT(head1)) {
-
-            cert1 = head1->cert;
-            if (cert1 == cert) {
-                continue;
-            }
-
-            if (SECITEM_CompareItem(&cert1->derIssuer, &cert->derSubject)
-                                      == SECEqual) {
-                break;
-            }
-        }
-
-        if (!CERT_LIST_END(head1, certs)) {
-            continue;
-        }
-
-        if((keyInfoCtx->flags & XMLSEC_KEYINFO_FLAGS_X509DATA_DONT_VERIFY_CERTS) == 0) {
-            /* it's important to set the usage here, otherwise no real verification
-             * is performed. */
-            status = CERT_VerifyCertificate(CERT_GetDefaultCertDB(),
-                                            cert, PR_FALSE,
-                                            certificateUsageEmailSigner,
-                                            timeboundary , NULL, NULL, NULL);
-            if(status == SECSuccess) {
-                break;
-            }
-        } else {
-            status = SECSuccess;
-            break;
+    /* do we need to verify anything at all? */
+    if((keyInfoCtx->flags & XMLSEC_KEYINFO_FLAGS_X509DATA_DONT_VERIFY_CERTS) != 0) {
+        good_certs = certs;
+    } else {
+        /* look through the certs and remove all revoked certs */
+        ret = xmlSecNssX509StoreRemoveRevokedCerts(ctx, certs, &good_certs, keyInfoCtx);
+        if((ret < 0) || (good_certs == NULL)) {
+            xmlSecInternalError("xmlSecNssX509StoreRemoveRevokedCerts",  xmlSecKeyDataStoreGetName(store));
+            CERT_DestroyCertList(good_certs);
+            return(NULL);
         }
     }
 
+    /* now go through all good certs we have and try to verify them */
+    for (cur = CERT_LIST_HEAD(good_certs); !CERT_LIST_END(cur, good_certs); cur = CERT_LIST_NEXT(cur)) {
+        cert = cur->cert;
+
+        /* if cert is the issuer of any other cert in the list, then it is
+         * to be skipped (note that we are using the bigger "certs" list instead of good_certs!) */
+        if(xmlSecNssX509StoreFindChildCert(cert, certs) != NULL) {
+            /* found a child, skip */
+            continue;
+        }
+
+        /* do we need to verify anything at all? */
+        if((keyInfoCtx->flags & XMLSEC_KEYINFO_FLAGS_X509DATA_DONT_VERIFY_CERTS) != 0) {
+            status = SECSuccess;
+            break;
+        }
+        /* it's important to set the usage here, otherwise no real verification
+         * is performed. */
+        status = CERT_VerifyCertificate(CERT_GetDefaultCertDB(), cert, PR_FALSE,
+                    certificateUsageEmailSigner,
+                    timeboundary , NULL, NULL, NULL);
+        if(status == SECSuccess) {
+            break;
+        }
+    }
+    /* SMALL HACK: we are using the fact that NSS implements certs as ref counted objects
+     * and CERT_DupCertificate() simply bumps the counter. Otherwise, the "cert"
+     * might belong to good_certs and will be destroyed here. But exactly the
+     * same pointer is in certs as well so we are good. Otherwise we will need to find
+     * a certificates in "certs" that matches "cert" and return that pointer instead.
+     */
+    if(good_certs != certs) {
+        CERT_DestroyCertList(good_certs);
+    }
     if (status == SECSuccess) {
         return (cert);
     }
@@ -288,25 +517,25 @@ xmlSecNssX509StoreVerify(xmlSecKeyDataStorePtr store, CERTCertList* certs,
             xmlSecOtherError2(XMLSEC_ERRORS_R_CERT_ISSUER_FAILED,
                 xmlSecKeyDataStoreGetName(store),
                 "subject=\"%s\"; reason=the issuer's cert is expired/invalid or not found",
-                xmlSecErrorsSafeString(cert->subjectName));
+                xmlSecErrorsSafeString(cert != NULL ? cert->subjectName : NULL));
             break;
         case SEC_ERROR_EXPIRED_CERTIFICATE:
             xmlSecOtherError2(XMLSEC_ERRORS_R_CERT_HAS_EXPIRED,
                 xmlSecKeyDataStoreGetName(store),
                 "subject=\"%s\"; reason=expired",
-                xmlSecErrorsSafeString(cert->subjectName));
+                xmlSecErrorsSafeString(cert != NULL ? cert->subjectName : NULL));
             break;
         case SEC_ERROR_REVOKED_CERTIFICATE:
             xmlSecOtherError2(XMLSEC_ERRORS_R_CERT_REVOKED,
                 xmlSecKeyDataStoreGetName(store),
                 "subject=\"%s\"; reason=revoked",
-                xmlSecErrorsSafeString(cert->subjectName));
+                xmlSecErrorsSafeString(cert != NULL ? cert->subjectName : NULL));
             break;
         default:
             xmlSecOtherError3(XMLSEC_ERRORS_R_CERT_VERIFY_FAILED,
                 xmlSecKeyDataStoreGetName(store),
                 "subject=\"%s\"; reason=%d",
-                xmlSecErrorsSafeString(cert->subjectName),
+                xmlSecErrorsSafeString(cert != NULL ? cert->subjectName : NULL),
                 err);
             break;
     }
@@ -369,6 +598,35 @@ xmlSecNssX509StoreAdoptCert(xmlSecKeyDataStorePtr store, CERTCertificate* cert,
     return(0);
 }
 
+
+/**
+ * xmlSecNssX509StoreAdoptCrl:
+ * @store:              the pointer to X509 key data store klass.
+ * @crl:                the pointer to NSS X509 CRL.
+ *
+ * Adds CRL to the store.
+ *
+ * Returns: 0 on success or a negative value if an error occurs.
+ */
+int
+xmlSecNssX509StoreAdoptCrl(xmlSecKeyDataStorePtr store, CERTSignedCrl * crl) {
+    xmlSecNssX509StoreCtxPtr ctx;
+    int ret;
+
+    xmlSecAssert2(xmlSecKeyDataStoreCheckId(store, xmlSecNssX509StoreId), -1);
+    xmlSecAssert2(crl != NULL, -1);
+
+    ctx = xmlSecNssX509StoreGetCtx(store);
+    xmlSecAssert2(ctx != NULL, -1);
+
+    ret = xmlSecNssX509CrlListAdoptCrl(&(ctx->crlsList), crl);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecNssX509CrlListAdoptCrl", xmlSecKeyDataStoreGetName(store));
+        return(-1);
+    }
+    return(0);
+}
+
 static int
 xmlSecNssX509StoreInitialize(xmlSecKeyDataStorePtr store) {
     xmlSecNssX509StoreCtxPtr ctx;
@@ -394,6 +652,10 @@ xmlSecNssX509StoreFinalize(xmlSecKeyDataStorePtr store) {
         CERT_DestroyCertList(ctx->certsList);
         ctx->certsList = NULL;
     }
+    if (ctx->crlsList != NULL) {
+        xmlSecNssX509CrlListDestroy(ctx->crlsList);
+        ctx->crlsList = NULL;
+    }
 
     memset(ctx, 0, sizeof(xmlSecNssX509StoreCtx));
 }
@@ -451,151 +713,63 @@ xmlSecNssGetCertName(const xmlChar * name) {
 }
 
 static CERTCertificate*
-xmlSecNssX509FindCert(CERTCertList* certsList, const xmlChar *subjectName,
-                      const xmlChar *issuerName, const xmlChar *issuerSerial,
-                      xmlSecByte * ski, xmlSecSize skiSize) {
-    CERTCertificate *cert = NULL;
-    CERTName *name = NULL;
-    SECItem *nameitem = NULL;
-    CERTCertListNode* head;
-    SECItem tmpitem;
-    SECStatus status;
-    PRArenaPool *arena = NULL;
-    int rv;
+xmlSecNssX509FindCert(CERTCertList* certsList, xmlSecNssX509FindCertCtxPtr findCertCtx) {
+    CERTCertDBHandle * certDb;
+    CERTCertificate * cert = NULL;
+    int ret;
 
     /* certsList can be NULL */
+    xmlSecAssert2(findCertCtx != NULL, NULL);
 
-    /* search by subject name if available */
-    if ((cert == NULL) && (subjectName != NULL)) {
-        name = xmlSecNssGetCertName(subjectName);
-        if (name == NULL) {
-            xmlSecInternalError2("xmlSecNssGetCertName", NULL,
-                                 "subject=%s",
-                                 xmlSecErrorsSafeString(subjectName));
-            goto done;
-        }
+    /* try to search in our list - NSS doesn't update it's cache correctly
+     * when new certs are added https://bugzilla.mozilla.org/show_bug.cgi?id=211051
+     */
+    if(certsList != NULL) {
+        CERTCertListNode* curCertNode;
 
-        if(arena == NULL) {
-            arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
-            if (arena == NULL) {
-                xmlSecNssError("PORT_NewArena", NULL);
-                goto done;
+        for(curCertNode = CERT_LIST_HEAD(certsList);
+            (cert == NULL) && !CERT_LIST_END(curCertNode, certsList) &&
+            (curCertNode != NULL) && (curCertNode->cert != NULL);
+            curCertNode = CERT_LIST_NEXT(curCertNode)
+        ) {
+            ret = xmlSecNssX509FindCertCtxMatch(findCertCtx, curCertNode->cert);
+            if(ret < 0) {
+                xmlSecInternalError("xmlSecNssX509FindCertCtxMatch", NULL);
+                return(NULL);
+            } else if(ret == 1) {
+                cert = CERT_DupCertificate(curCertNode->cert);
+                if(cert == NULL) {
+                    xmlSecNssError("CERT_DupCertificate", NULL);
+                    return(NULL);
+                }
+                return(cert);
             }
         }
+    }
 
-        nameitem = SEC_ASN1EncodeItem(arena, NULL, (void *)name,
-                                      SEC_ASN1_GET(CERT_NameTemplate));
-        if (nameitem == NULL) {
-            xmlSecNssError("SEC_ASN1EncodeItem", NULL);
-            goto done;
-        }
+    /* search in the NSS DB */
+    certDb = CERT_GetDefaultCertDB();
+    if(certDb == NULL) {
+        xmlSecNssError("CERT_GetDefaultCertDB(ski)", NULL);
+        return(NULL);
+    }
 
-        cert = CERT_FindCertByName(CERT_GetDefaultCertDB(), nameitem);
+    /* search by subject name if available */
+    if ((cert == NULL) && (findCertCtx->subjectNameItem != NULL)) {
+        cert = CERT_FindCertByName(certDb, findCertCtx->subjectNameItem);
     }
 
     /* search by issuer name+serial if available */
-    if((cert == NULL) && (issuerName != NULL) && (issuerSerial != NULL)) {
-        CERTIssuerAndSN issuerAndSN;
-        PRUint64 issuerSN = 0;
-
-        name = xmlSecNssGetCertName(issuerName);
-        if (name == NULL) {
-            xmlSecInternalError2("xmlSecNssGetCertName", NULL,
-                                 "issuer=%s",
-                                 xmlSecErrorsSafeString(issuerName));
-            goto done;
-        }
-
-        if(arena == NULL) {
-            arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
-            if (arena == NULL) {
-                xmlSecNssError("PORT_NewArena", NULL);
-                goto done;
-            }
-        }
-
-        nameitem = SEC_ASN1EncodeItem(arena, NULL, (void *)name,
-                                      SEC_ASN1_GET(CERT_NameTemplate));
-        if (nameitem == NULL) {
-            xmlSecNssError("SEC_ASN1EncodeItem", NULL);
-            goto done;
-        }
-
-        memset(&issuerAndSN, 0, sizeof(issuerAndSN));
-
-        issuerAndSN.derIssuer.data = nameitem->data;
-        issuerAndSN.derIssuer.len = nameitem->len;
-
-        /* TBD: serial num can be arbitrarily long */
-        if(PR_sscanf((char *)issuerSerial, "%llu", &issuerSN) != 1) {
-            xmlSecNssError("PR_sscanf(issuerSerial)", NULL);
-            SECITEM_FreeItem(&issuerAndSN.serialNumber, PR_FALSE);
-            goto done;
-        }
-
-        rv = xmlSecNssNumToItem(&issuerAndSN.serialNumber, issuerSN);
-        if(rv <= 0) {
-            xmlSecInternalError("xmlSecNssNumToItem(serialNumber)", NULL);
-            SECITEM_FreeItem(&issuerAndSN.serialNumber, PR_FALSE);
-            goto done;
-        }
-
-        cert = CERT_FindCertByIssuerAndSN(CERT_GetDefaultCertDB(), &issuerAndSN);
-        SECITEM_FreeItem(&issuerAndSN.serialNumber, PR_FALSE);
+    if((cert == NULL) && (findCertCtx->issuerAndSNInitialized == 1)) {
+        cert = CERT_FindCertByIssuerAndSN(certDb, &(findCertCtx->issuerAndSN));
     }
 
     /* search by SKI if available */
-    if((cert == NULL) && (ski != NULL) && (skiSize > 0)) {
-        SECItem subjKeyID;
-
-        memset(&subjKeyID, 0, sizeof(subjKeyID));
-        subjKeyID.data = ski;
-        XMLSEC_SAFE_CAST_SIZE_TO_UINT(skiSize, subjKeyID.len, goto done, NULL);
-
-        cert = CERT_FindCertBySubjectKeyID(CERT_GetDefaultCertDB(),
-                                           &subjKeyID);
-
-        /* try to search in our list - NSS doesn't update it's cache correctly
-         * when new certs are added https://bugzilla.mozilla.org/show_bug.cgi?id=211051
-         */
-        if((cert == NULL) && (certsList != NULL)) {
-            for(head = CERT_LIST_HEAD(certsList);
-                (cert == NULL) && !CERT_LIST_END(head, certsList) &&
-                (head != NULL) && (head->cert != NULL);
-                head = CERT_LIST_NEXT(head)
-            ) {
-
-                memset(&tmpitem, 0, sizeof(tmpitem));
-                status = CERT_FindSubjectKeyIDExtension(head->cert, &tmpitem);
-                if (status != SECSuccess)  {
-                    xmlSecNssError("CERT_FindSubjectKeyIDExtension(ski)", NULL);
-                    SECITEM_FreeItem(&tmpitem, PR_FALSE);
-                    goto done;
-                }
-
-                if((tmpitem.len == subjKeyID.len) &&
-                   (memcmp(tmpitem.data, subjKeyID.data, subjKeyID.len) == 0)
-                ) {
-                    cert = CERT_DupCertificate(head->cert);
-                    if(cert == NULL) {
-                        xmlSecNssError("CERT_DupCertificate", NULL);
-                        SECITEM_FreeItem(&tmpitem, PR_FALSE);
-                        goto done;
-                    }
-                }
-                SECITEM_FreeItem(&tmpitem, PR_FALSE);
-            }
-        }
-    }
-
-done:
-    if (arena != NULL) {
-        PORT_FreeArena(arena, PR_FALSE);
-    }
-    if (name != NULL) {
-        CERT_DestroyName(name);
+    if((cert == NULL) && (findCertCtx->skiItem.data != NULL) && (findCertCtx->skiItem.len > 0)) {
+        cert = CERT_FindCertBySubjectKeyID(certDb, &(findCertCtx->skiItem));
     }
 
+    /* done */
     return(cert);
 }
 
@@ -741,7 +915,7 @@ xmlSecNssX509NameStringRead(const xmlSecByte **in, xmlSecSize *inSize,
                     xmlSecInvalidDataError("two hex digits expected", NULL);
                     return(-1);
                 }
-                outCh = (xmlSecByte)(xmlSecGetHex(inCh) * 16 + xmlSecGetHex(inCh2));
+                outCh = xmlSecFromHex2(inCh, inCh2);
             } else {
                 outCh = inCh;
             }
@@ -771,12 +945,13 @@ xmlSecNssX509NameStringRead(const xmlSecByte **in, xmlSecSize *inSize,
 
 /* code lifted from NSS */
 static int
-xmlSecNssNumToItem(SECItem *it, PRUint64 ui)
+xmlSecNssNumToItem(PLArenaPool *arena, SECItem *it, PRUint64 ui)
 {
     unsigned char bb[9];
     unsigned int bb_len, zeros_len;
     int res;
 
+    xmlSecAssert2(arena != NULL, -1);
     xmlSecAssert2(it != NULL, -1);
 
     bb[0] = 0; /* important: we should have 0 at the beginning! */
@@ -799,7 +974,7 @@ xmlSecNssNumToItem(SECItem *it, PRUint64 ui)
     }
 
     it->len = bb_len - (zeros_len - 1);
-    it->data = (unsigned char *)PORT_Alloc(it->len * sizeof(bb[0]));
+    it->data = (unsigned char *)PORT_ArenaAlloc(arena, it->len * sizeof(bb[0]));
     if (it->data == NULL) {
         it->len = 0;
         return (-1);
@@ -810,6 +985,341 @@ xmlSecNssNumToItem(SECItem *it, PRUint64 ui)
 
     return(res);
 }
+
+xmlSecKeyPtr
+xmlSecNssX509FindKeyByValue(xmlSecPtrListPtr keysList, xmlSecKeyX509DataValuePtr x509Value) {
+    xmlSecNssX509FindCertCtx findCertCtx;
+    xmlSecSize keysListSize, ii;
+    xmlSecKeyPtr res = NULL;
+    int ret;
+
+    xmlSecAssert2(keysList != NULL, NULL);
+    xmlSecAssert2(x509Value != NULL, NULL);
+
+    ret = xmlSecNssX509FindCertCtxInitializeFromValue(&findCertCtx, x509Value);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecNssX509FindCertCtxInitializeFromValue", NULL);
+        xmlSecNssX509FindCertCtxFinalize(&findCertCtx);
+        return(NULL);
+    }
+
+    keysListSize = xmlSecPtrListGetSize(keysList);
+    for(ii = 0; ii < keysListSize; ++ii) {
+        xmlSecKeyPtr key;
+        xmlSecKeyDataPtr keyData;
+        CERTCertificate* keyCert;
+
+        /* get key's cert from x509 key data */
+        key = (xmlSecKeyPtr)xmlSecPtrListGetItem(keysList, ii);
+        if(key == NULL) {
+            continue;
+        }
+        keyData = xmlSecKeyGetData(key, xmlSecNssKeyDataX509Id);
+        if(keyData == NULL) {
+            continue;
+        }
+        keyCert = xmlSecNssKeyDataX509GetKeyCert(keyData);
+        if(keyCert == NULL) {
+            continue;
+        }
+
+        /* does it match? */
+        ret = xmlSecNssX509FindCertCtxMatch(&findCertCtx, keyCert);
+        if(ret < 0) {
+            xmlSecInternalError("xmlSecNssX509FindCertCtxMatch", NULL);
+            xmlSecNssX509FindCertCtxFinalize(&findCertCtx);
+            return(NULL);
+        } else if(ret == 1) {
+            res = key;
+            break;
+        }
+    }
+
+    /* done */
+    xmlSecNssX509FindCertCtxFinalize(&findCertCtx);
+    return(res);
+}
+
+/***********************************************************************************
+ *
+ * xmlSecNssX509FindCertCtx
+ *
+ **********************************************************************************/
+SECOidTag
+xmlSecNssX509GetDigestFromAlgorithm(const xmlChar* href) {
+    /* use SHA256 by default */
+    if(href == NULL) {
+#ifndef XMLSEC_NO_SHA256
+        return(SEC_OID_SHA256);
+#else  /* XMLSEC_NO_SHA256 */
+        xmlSecOtherError2(XMLSEC_ERRORS_R_INVALID_ALGORITHM, NULL,
+            "sha256 disabled and href=%s", xmlSecErrorsSafeString(href));
+        return(SEC_OID_UNKNOWN);
+#endif /* XMLSEC_NO_SHA256 */
+    } else
+
+#ifndef XMLSEC_NO_SHA1
+    if(xmlStrcmp(href, xmlSecHrefSha1) == 0) {
+        return(SEC_OID_SHA1);
+    } else
+#endif /* XMLSEC_NO_SHA1 */
+
+#ifndef XMLSEC_NO_SHA224
+    if(xmlStrcmp(href, xmlSecHrefSha224) == 0) {
+        return(SEC_OID_SHA224);
+    } else
+#endif /* XMLSEC_NO_SHA224 */
+
+#ifndef XMLSEC_NO_SHA256
+    if(xmlStrcmp(href, xmlSecHrefSha256) == 0) {
+        return(SEC_OID_SHA256);
+    } else
+#endif /* XMLSEC_NO_SHA256 */
+
+#ifndef XMLSEC_NO_SHA384
+    if(xmlStrcmp(href, xmlSecHrefSha384) == 0) {
+        return(SEC_OID_SHA384);
+    } else
+#endif /* XMLSEC_NO_SHA384 */
+
+#ifndef XMLSEC_NO_SHA512
+    if(xmlStrcmp(href, xmlSecHrefSha512) == 0) {
+        return(SEC_OID_SHA512);
+    } else
+#endif /* XMLSEC_NO_SHA512 */
+
+    {
+        xmlSecOtherError2(XMLSEC_ERRORS_R_INVALID_ALGORITHM, NULL,
+            "href=%s", xmlSecErrorsSafeString(href));
+        return(SEC_OID_UNKNOWN);
+    }
+}
+
+
+int xmlSecNssX509FindCertCtxInitialize(xmlSecNssX509FindCertCtxPtr ctx,
+    const xmlChar *subjectName,
+    const xmlChar *issuerName, const xmlChar *issuerSerial,
+    xmlSecByte * ski, xmlSecSize skiSize
+) {
+    int ret;
+
+    xmlSecAssert2(ctx != NULL, -1);
+
+    memset(ctx, 0, sizeof(*ctx));
+
+    /* ski (easy first) */
+    if((ski != NULL) && (skiSize > 0)) {
+        ctx->skiItem.type = siBuffer;
+        ctx->skiItem.data = ski;
+        XMLSEC_SAFE_CAST_SIZE_TO_UINT(skiSize, ctx->skiItem.len, return(-1), NULL);
+    }
+
+    ctx->arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
+    if (ctx->arena == NULL) {
+        xmlSecNssError("PORT_NewArena", NULL);
+        xmlSecNssX509FindCertCtxFinalize(ctx);
+        return(-1);
+    }
+
+    /* subject name */
+    if(subjectName != NULL) {
+        ctx->subjectName = xmlSecNssGetCertName(subjectName);
+        if(ctx->subjectName == NULL) {
+            xmlSecInternalError2("xmlSecNssGetCertName", NULL,
+                "subjectName=%s", xmlSecErrorsSafeString(subjectName));
+            xmlSecNssX509FindCertCtxFinalize(ctx);
+            return(-1);
+        }
+        ctx->subjectNameItem = SEC_ASN1EncodeItem(ctx->arena, NULL, (void *)ctx->subjectName , SEC_ASN1_GET(CERT_NameTemplate));
+        if (ctx->subjectNameItem == NULL) {
+            xmlSecNssError2("SEC_ASN1EncodeItem(subjectName)", NULL,
+                "subjectName=%s", xmlSecErrorsSafeString(subjectName));
+            xmlSecNssX509FindCertCtxFinalize(ctx);
+            return(-1);
+        }
+    }
+
+    /* issuer name + serial  */
+    if((issuerName != NULL) && (issuerSerial != NULL)) {
+        memset(&ctx->issuerAndSN, 0, sizeof(ctx->issuerAndSN));
+
+        ctx->issuerName = xmlSecNssGetCertName(issuerName);
+        if(ctx->issuerName == NULL) {
+            xmlSecInternalError2("xmlSecNssGetCertName", NULL,
+                "issuerName=%s", xmlSecErrorsSafeString(issuerName));
+            xmlSecNssX509FindCertCtxFinalize(ctx);
+            return(-1);
+        }
+        ctx->issuerNameItem = SEC_ASN1EncodeItem(ctx->arena, NULL, (void *)ctx->issuerName , SEC_ASN1_GET(CERT_NameTemplate));
+        if (ctx->issuerNameItem == NULL) {
+            xmlSecNssError2("SEC_ASN1EncodeItem(issuerName)", NULL,
+                "issuerName=%s", xmlSecErrorsSafeString(issuerName));
+            xmlSecNssX509FindCertCtxFinalize(ctx);
+            return(-1);
+        }
+        ctx->issuerAndSN.derIssuer.type = ctx->issuerNameItem->type;
+        ctx->issuerAndSN.derIssuer.data = ctx->issuerNameItem->data;
+        ctx->issuerAndSN.derIssuer.len  = ctx->issuerNameItem->len;
+
+        /* TBD: serial num can be arbitrarily long */
+        if(PR_sscanf((char *)issuerSerial, "%llu", &(ctx->issuerSN)) != 1) {
+            xmlSecNssError("PR_sscanf(issuerSerial)", NULL);
+            xmlSecNssX509FindCertCtxFinalize(ctx);
+            return(-1);
+        }
+        ret = xmlSecNssNumToItem(ctx->arena, &(ctx->issuerAndSN.serialNumber), ctx->issuerSN);
+        if(ret <= 0) {
+            xmlSecInternalError("xmlSecNssNumToItem(serialNumber)", NULL);
+            xmlSecNssX509FindCertCtxFinalize(ctx);
+            return(-1);
+        }
+        ctx->issuerAndSNInitialized = 1;
+    }
+
+    /* done! */
+    return(0);
+}
+
+int
+xmlSecNssX509FindCertCtxInitializeFromValue(xmlSecNssX509FindCertCtxPtr ctx, xmlSecKeyX509DataValuePtr x509Value) {
+    int ret;
+
+    xmlSecAssert2(ctx != NULL, -1);
+    xmlSecAssert2(x509Value != NULL, -1);
+
+    ret = xmlSecNssX509FindCertCtxInitialize(ctx,
+                x509Value->subject,
+                x509Value->issuerName, x509Value->issuerSerial,
+                xmlSecBufferGetData(&(x509Value->ski)), xmlSecBufferGetSize(&(x509Value->ski))
+    );
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecNssX509FindCertCtxInitialize", NULL);
+        xmlSecNssX509FindCertCtxFinalize(ctx);
+        return(-1);
+    }
+
+    if((!xmlSecBufferIsEmpty(&(x509Value->digest))) && (x509Value->digestAlgorithm != NULL)) {
+        xmlSecSize digestSize;
+
+        ctx->digestValue = xmlSecBufferGetData(&(x509Value->digest));
+        digestSize = xmlSecBufferGetSize(&(x509Value->digest));
+        XMLSEC_SAFE_CAST_SIZE_TO_UINT(digestSize, ctx->digestLen, return(-1), NULL);
+
+        ctx->digestAlg = xmlSecNssX509GetDigestFromAlgorithm(x509Value->digestAlgorithm);
+        if(ctx->digestAlg == SEC_OID_UNKNOWN) {
+            xmlSecInternalError("xmlSecNssX509GetDigestFromAlgorithm", NULL);
+            xmlSecNssX509FindCertCtxFinalize(ctx);
+            return(-1);
+        }
+    }
+
+    return(0);
+}
+
+void
+xmlSecNssX509FindCertCtxFinalize(xmlSecNssX509FindCertCtxPtr ctx) {
+    xmlSecAssert(ctx != NULL);
+
+    if(ctx->subjectName != NULL) {
+        CERT_DestroyName(ctx->subjectName);
+    }
+    if(ctx->issuerName != NULL) {
+        CERT_DestroyName(ctx->issuerName);
+    }
+    if (ctx->arena != NULL) {
+        PORT_FreeArena(ctx->arena, PR_FALSE);
+    }
+    memset(ctx, 0, sizeof(*ctx));
+}
+
+/* returns 1 for match, 0 for no match, and a negative value if an error occurs */
+int
+xmlSecNssX509FindCertCtxMatch(xmlSecNssX509FindCertCtxPtr ctx, CERTCertificate* cert) {
+    SECStatus status;
+
+    xmlSecAssert2(ctx != NULL, -1);
+    xmlSecAssert2(cert != NULL, -1);
+
+
+    /* subject name */
+    if(ctx->subjectNameItem != NULL) {
+        if (SECITEM_ItemsAreEqual(&(cert->derSubject), ctx->subjectNameItem)) {
+            /* found a match */
+            return(1);
+        } else {
+            /* no match */
+            return(0);
+        }
+    }
+
+    /* issuer name + serial */
+    if(ctx->issuerAndSNInitialized != 0) {
+        if (
+            SECITEM_ItemsAreEqual(&(cert->derIssuer),  &(ctx->issuerAndSN.derIssuer)) &&
+            SECITEM_ItemsAreEqual(&(cert->serialNumber),  &(ctx->issuerAndSN.serialNumber))
+        ) {
+            /* found a match */
+            return(1);
+        } else {
+            /* no match */
+            return(0);
+        }
+    }
+
+    /* ski */
+    if( (ctx->skiItem.data != NULL) && (ctx->skiItem.len > 0)) {
+        SECItem tmpitem;
+
+        memset(&tmpitem, 0, sizeof(tmpitem));
+        status = CERT_FindSubjectKeyIDExtension(cert, &tmpitem);
+        if (status != SECSuccess)  {
+            xmlSecNssError("CERT_FindSubjectKeyIDExtension(ski)", NULL);
+            return(-1);
+        }
+
+        if((tmpitem.len != ctx->skiItem.len) || (memcmp(tmpitem.data, ctx->skiItem.data, ctx->skiItem.len) != 0)) {
+            /* no match */
+            SECITEM_FreeItem(&tmpitem, PR_FALSE);
+            return(0);
+        }
+        SECITEM_FreeItem(&tmpitem, PR_FALSE);
+
+        /* found a match */
+        return(1);
+    }
+
+    /* cert digest */
+    if(
+         (ctx->digestAlg != SEC_OID_UNKNOWN) && (ctx->digestValue != NULL) && (ctx->digestLen > 0) &&
+        (cert->derCert.type == siBuffer) && (cert->derCert.data != NULL) && (cert->derCert.len > 0)
+    ) {
+        xmlSecByte digest[XMLSEC_NSS_MAX_DIGEST_SIZE];
+        unsigned int digestLen;
+
+        digestLen = HASH_ResultLenByOidTag(ctx->digestAlg);
+        if((digestLen == 0) || (digestLen > sizeof(digest))) {
+            xmlSecNssError3("HASH_ResultLenByOidTag", NULL,
+                "digestAlgOid=%d; len=%u", (int)ctx->digestAlg, digestLen);
+            return(-1);
+        }
+        status = PK11_HashBuf(ctx->digestAlg, digest, cert->derCert.data, (PRInt32)cert->derCert.len);
+        if (status != SECSuccess) {
+            xmlSecNssError2("PK11_HashBuf(cert->derCert)", NULL,
+                "digestAlgOid=%d", (int)ctx->digestAlg);
+            return(-1);
+        }
+
+        if((digestLen != ctx->digestLen) || (memcmp(digest, ctx->digestValue, ctx->digestLen) != 0)) {
+            /* no match */
+            return(0);
+        }
+
+        /* found a match */
+        return(1);
+    }
+
+
+    return(0);
+}
+
 #endif /* XMLSEC_NO_X509 */
-
-
